/* linux/arch/arm/mach-mb86hxx/include/mach/sdhci.h
 *  *
 *   * Copyright 2011 Fujitsu FSDC.
 *       *
 *        * MB86H61 Platform - SDIO function data definitions
 *         *
 *          * This program is free software; you can redistribute it and/or modify
 *           * it under the terms of the GNU General Public License version 2 as
 *            * published by the Free Software Foundation.
 *            */

#ifndef __PLAT_MB86H61_SDIO_H
#define __PLAT_MB86H61_SDIO_H __FILE__
 
 struct platform_device;
 struct mmc_host;
 struct mmc_card;
 struct mmc_ios;
#if defined(CONFIG_MMC_MB86H61_SDIO) || defined(CONFIG_MMC_MB86H61_SDIO_MODULE) 
 /**
  * struct mb86h6x_sdhci_platdata() - Platform device data for Fujitsu mb86h6x sdio controller
  * @max_width: The maximum number of data bits supported.
  * @host_caps: Standard MMC host capabilities bit field.
  * @cfg_gpio: Configure the GPIO for a specific card bit-width
  * @cfg_card: Configure the interface for a specific card and speed. This
  * 		   is necessary the controllers and/or GPIO blocks require the
  * 	   changing of driver-strength and other controls dependant on
  * 	   the card and speed of operation.
  *
  * Initialisation data specific to either the machine or the platform
  * for the device driver to use or call-back when configuring gpio or
  * card speed information.
 */
 struct mb86h6x_sdhci_platdata {
	 unsigned int	 max_width;
	 unsigned int	 host_caps;
 
	 char		 **clocks;	 /* set of clock sources */
 
	 void	 (*cfg_gpio)(struct platform_device *dev, int width);
	 void	 (*cfg_card)(struct platform_device *dev,
				 void __iomem *regbase,
				 struct mmc_ios *ios,
				 struct mmc_card *card);
 };
 
 /**
  * s3c_sdhci0_set_platdata - Set platform data for S3C SDHCI device.
  * @pd: Platform data to register to device.
  *
  * Register the given platform data for use withe S3C SDHCI device.
  * The call will copy the platform data, so the board definitions can
  * make the structure itself __initdata.
  */
// extern void s3c_sdhci0_set_platdata(struct mb86h6x_sdhci_platdata *pd);
// extern void s3c_sdhci1_set_platdata(struct mb86h6x_sdhci_platdata *pd);
 
 /* Default platform data, exported so that per-cpu initialisation can
  * set the correct one when there are more than one cpu type selected.
 */
 
 extern struct mb86h6x_sdhci_platdata mb86h61_hsmmc0_def_platdata;
 
 /* Helper function availablity */
 
 extern void mb86h6x_setup_sdhci0_cfg_gpio(struct platform_device *, int w);
 
 /* mb86h6x SDHCI setup */
 


 extern char *mb86h6x_hsmmc_clksrcs[4];
 
 extern void mb86h6x_setup_sdhci_cfg_card(struct platform_device *dev,
					  void __iomem *r,
					  struct mmc_ios *ios,
					  struct mmc_card *card);
 
 static inline void mb86h6x_default_sdhci0(void)
 {
	 mb86h61_hsmmc0_def_platdata.clocks = mb86h6x_hsmmc_clksrcs;
	 mb86h61_hsmmc0_def_platdata.cfg_gpio = mb86h6x_setup_sdhci0_cfg_gpio;//s3c64xx_setup_sdhci0_cfg_gpio;
	 mb86h61_hsmmc0_def_platdata.cfg_card = mb86h6x_setup_sdhci_cfg_card;
 }
 
 
 
 
#else
 static inline void mb86h6x_default_sdhci0(void) { }

#endif /* CONFIG_S3C6400_SETUP_SDHCI */
 
 
 
 
#endif /* __PLAT_MB86H61_SDIO_H */

