/*
 *  linux/arch/arm/mach-mb86hxx/io_addr.c
 *
 * Copyright (C) 2008-2010 FUJITSU MICROELECTRONICS LIMITED
 * Copyright (C) 2010-2011 FUJITSU SEMICONDUCTOR LIMITED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/string.h>
#include <mach/hardware.h>
#include <asm/hardware/uengine.h>
#include <asm/io.h>
#include <mach/memory.h>

unsigned long io_phys_to_virt(unsigned long x)
{
	if ((x & 0xc0000000) == 0xc0000000) {	/* PA: 0xc000_0000 - 0xffff_ffff */
		if ((x & 0xe0000000) == 0xe0000000)	/* PA: 0xe000_0000 - 0xefff_fffff */
		{
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC)
			if ((x & 0x4000000) == 0x4000000) { /* PA: 0xe400_0000  Ethernet (h60)*/
				x = 0xfffe0000 + ((x << 16) >> 16);
				return x;
			} 
			else { /* PA: 0xe000_0000 0xe2ff_fffff */  /* VIC */
				x = 0xfffef000 + ((x << 20) >> 20);
				return x;
			}
            #elif defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
			if ((x & 0x8000000) == 0x8000000) { /* PA: 0xe800_0000  NAND flash (h61)*/
				x = 0xf9000000 + ((x << 16) >> 16);
				return x;
			}
			else if ((x & 0x2000000) == 0x2000000) {
				   /* PA: 0xe200_0000  ethernet (h61)*/
				x = 0xfffe2000 + ((x << 16) >> 16);
			}
			else { /* PA: 0xe000_0000  VIC  (h61)*/
				   /* PA: 0xe400_0000  SATA (h61)*/
				   /* PA: 0xe600_0000  HDMAC (h61)*/
				   /* PA: 0xea00_0000  NAND CFG (h61)*/
				   /* PA: 0xec00_0000  SDIO (h61)*/
				x = 0xfffe0000 + ((x << 4) >> 16) + ((x<<20)>>20);
				return x;
			}
            #endif
		} 
		else if ((x & 0xd0000000) == 0xd0000000) {  /* PA: 0xd000_0000 - 0xdff_fffff */
				/* PA: 0xD000_0000 */          /* SSP0(h60)/I2C0(h61), I2C1, I2C2, 
				                                  SSP1(h60)/SSP0(h61), EFUSE(h60), PIRLO,
				                                  SCHED, RAMIFS,RAMIFV */ 
			x = 0xff650000 + ((x << 4) >> 16) + ((x << 16) >> 16);
			return x;
		} 
		else {	/* PA: 0xc000_0000 - 0xcfff_ffff */

			if ((x & 0x0c000000) == 0x0c000000) {  
				/* PA: 0xCC00_0000 - 0xCC00_0FFF */ /* AUD output */
				/* PA: 0xCD00_0000 - 0xCD00_0FFF */ /* UART0 (h61) */
				/* PA: 0xCE00_0000 - 0xCE00_0FFF */ /* UART1 */
				/* PA: 0xCF00_0000 - 0xCF00_0FFF */ /* TIMER0/1/2, IRC */
				x = 0xff640000 + ((x << 4) >> 16) +
				    ((x << 20) >> 20);
				return x;
			} 
			else if ((x & 0x0b000000) == 0x0b000000) {
				/* PA: 0xCB00_0000 - 0xCBFF_FFFF */ /* VIDOUT */
				x = 0xff610000 + ((x << 12) >> 12);
				return x;
			} 
			else if ((x & 0x0a000000) == 0x0a000000) {
				/* PA: 0xCA00_0000 - 0xCAFF_FFFF */ /* CLKPWR */
				x = 0xff60a000 + ((x << 20) >> 20);
				return x;
			} 

            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
			else if ((x & 0x09000000) == 0x09000000) {
				/* PA: 0xC900_0000 - 0xC9FF_FFFF */ /* CEC (h61) */
				x = 0xf9000000 + ((x << 20) >> 20);
				return x;
			} 
            #endif
			else if ((x & 0x08000000) == 0x08000000) {
				/* PA: 0xC800_0000 - 0xC8FF_FFFF */ /* TSD */
				x = 0xfe908000 + ((x << 8) >> 8);
				return x;
			} 
			else if ((x & 0x07000000) == 0x07000000) {
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
				/* PA: 0xC700_1000 - 0xC700_1FFF */ /* USB PHY 0 (h61) */
				/* PA: 0xC700_3000 - 0xC700_3FFF */ /* USB PHY 1 (h61) */
				if ((x & 0x1000) == 0x1000) {
					x = 0xf9000000 + ((x << 16) >> 16);
					return x;
				}
				else {
            #endif
				/* PA: 0xC700_0000 - 0xC700_07FF */ /* DMA */
				/* PA: 0xC700_0800 - 0xC7FF_FFFF */ /* USB */
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC)
				x = 0xfe906000 + ((x << 20) >> 20);
            #else
				x = 0xfe907000 + ((x << 20) >> 20); //changed for DMA & USB mapping by (shenmeng)
            #endif
				return x;
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
				}
            #endif
			} 
			else {
				/* PA: 0xC000_0000 - 0xC0FF_FFFF */ /* sflash */
				/* PA: 0xC100_0000 - 0xC1FF_FFFF */ /* FPC& IR (h60)/ FPC (h61)   */
				/* PA: 0xC200_0000 - 0xC2FF_FFFF */ /* UART0(h60) / IR (h61) */
				/* PA: 0xC300_0000 - 0xC3FF_FFFF */ /* GPIO  */
				/* PA: 0xC400_0000 - 0xC4FF_FFFF */ /* I2C0 (h60) / EFUSE (h61) */
				/* PA: 0xC500_0000 - 0xC5FF_FFFF */ /* ICC   */
				/* PA: 0xC600_0000 - 0xC6FF_FFFF */ /* R2R   */
				x = 0xfe900000 + ((x << 4) >> 16) +
				    ((x << 20) >> 20);
				return x;
			}
		}
	} 
	else {/* PA: 0x0000_0000 - 0xbfff_ffff */
		/* PA: 0x8000_0000 */ /* UPI */
		if ((x & 0x80000000) == 0x80000000) {
			x += 0x60000000;
			return x;
		}
		/* PA: 0x6000_0000 */ 
		if ((x & 0x60000000) == 0x60000000) { /* VIDDEC */
		  /* PA: 0x610e_2000 -> 0xf81d2000 */
		  if ((x & 0x610e0000) == 0x610e0000) { /* PINT */
			x += 0x970f0000;
			return x;
		}
			x += 0x98100000;
			return x;
		}
		/* PA: 0x4000_0000 */
		if ((x & 0x40000000) == 0x40000000) { /* DDR2-V */
            #if (CONFIG_MEM_SIZE_RAMIFS != 0)
			x += 0x98000000;  /* --> 0xD800_0000 */
            #else
			if (x < (0x40000000 + CONFIG_FIRM_AREA_SIZE)) {
				x += 0x98000000 - CONFIG_FIRM_AREA_SIZE;  /* --> 0xD800_0000 - CONFIG_FIRM_AREA_SIZE */
			}
			else if (x >= (0x40000000 + CONFIG_MEM_SIZE_RAMIFV - CONFIG_FIRM_FRAME_BUFFER_SIZE)) {
				x += 0x98000000 - CONFIG_MEM_SIZE_RAMIFV + CONFIG_FIRM_FRAME_BUFFER_SIZE;  /* --> 0xD800_0000 */
			}
			else {
				/* kernel (without vmalloc area) */
				x += (PAGE_OFFSET - PHYS_OFFSET);
			}
            #endif
			return x;
		}
		/* PA: 0x2000_0000 */
            #ifndef CONFIG_MB86HXX_LARGEMEM
		if ((x >= MB86HXX_BUF_BASE) && ((MB86HXX_DDR1_BASE + PHYS_SIZE) > x)) {
			x += (0xb0000000 + (0x8000000 - PHYS_SIZE));
			return x;
		}
            #endif /* CONFIG_MB86HXX_LARGEMEM */
		if ((x & 0x20000000) == 0x20000000) { /* DDR2-S */
			if (x < (0x20000000 + CONFIG_FIRM_AREA_SIZE)) {
				x += 0xb8000000 - CONFIG_FIRM_AREA_SIZE; /* --> 0xD800_0000 - CONFIG_FIRM_AREA_SIZE */
			}
			else {
				/* kernel (without vmalloc area) */
				x += (PAGE_OFFSET - PHYS_OFFSET);
			}
			return x;
		}
		/* PA: 0x0200_0000 */
		if ((x & 0x2000000) == 0x2000000) {  /* SRAM */
			x += 0xfc8f8000;
			return x;
		}
		/* PA: 0x0000_0000 */
		if ((x & 0x0) == 0x0) { /* ROM */
			x += 0xfe8f0000;
			return x;
		}
	}
	return x;
}

unsigned long io_virt_to_phys(unsigned long x)
{
	if ((x & 0xf0000000) == 0xf0000000) {

		if ((x & 0xfff00000) == 0xfff00000) {
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC)
			/* VA: 0xfffe_0000 */
			if ((x & 0xef000) == 0xef000) {  /* VIC */
				x = 0xe0000000 + ((x << 20) >> 20);
				return x;
			} else {
				x = 0xe4000000 + ((x << 16) >> 16); /* ethernet(h60) */
				return x;
			}
            #endif

            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
			/* VA: 0xfffe_2000 */
			if ((x & 0xe2000) == 0xe2000) {  /* etherenet (h61) */
				x = 0xe2000000 + ((x << 19) >> 19);
				return x;
			}
			else {
				x = 0xe0000000 + ((x << 12) >> 28) + ((x << 20) >> 20); /* (h61) */
			}
            #endif

		} else if ((x & 0xff600000) == 0xff600000) {

			/* VA: 0xff65_0000 */
			if ((x & 0xff650000) == 0xff650000) { /* SSP0(h60)/I2C0(h61), I2C1, I2C2, 
				                                 SSP1(h60)/SSP0(h61), EFUSE(h60)/SSP1(h61), 
				                                 HDMI link (h61), PIRLO, SCHED, RAMIFS,RAMIFV */
				x = 0xd0000000 + (((x >> 12) << 28) >> 4) +
				    ((x << 20) >> 20);
				return x;
			}
			/* VA: 0xff64_c000 */
			else if ((x & 0xff64f000) == 0xff64c000) { /* AUD */
				x = 0xc0000000 + (((x >> 12) << 28) >> 4) +
				    ((x << 20) >> 20);
				return x;
			}

            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
			/* VA: 0xff64_d000 */
			else if ((x & 0xff64f000) == 0xff64d000) { /* UART0 (h61) */
				x = 0xc0000000 + (((x >> 12) << 28) >> 4) +
				    ((x << 20) >> 20);
				return x;
			}
			/* VA: 0xff64_e000 */
			else if ((x & 0xff64f000) == 0xff64e000) { /* UART1 (h61) */
				x = 0xc0000000 + (((x >> 12) << 28) >> 4) +
				    ((x << 20) >> 20);
				return x;
			}
			/* VA: 0xff64_f000 */
			else if ((x & 0xff64f000) == 0xff64f000) { /* IRQ,timer0/1/2 (h61) */
				x = 0xc0000000 + (((x >> 12) << 28) >> 4) +
				    ((x << 20) >> 20);
				return x;
			}
            #endif
			/* VA: 0xff61_0000 - 0xff6f_ffff */
			else if ((x & 0xff6f0000) >= 0xff610000) { /* VIDOUT */
				x -= 0x34610000;
				return x;
			}
			/* VA: 0xff60_a000 */
			else if ((x & 0xff60a000) == 0xff60a000) { /* CLKPWR */
				x = 0xca000000 + ((x << 20) >> 20);
				return x;
			} else { /* tsd? */
				x -= 0x36908000;
				return x;
			}
		}
		/* VA: 0xfe90_8000 */
		else if (x >= 0xfe908000) { /* TSD */
			x -= 0x36908000;
			return x;
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC)
		} else if ((x & 0xfe9f6000) == 0xfe906000) { /* USB */
            #else
		} else if ((x & 0xfe9f7000) == 0xfe907000) { /* DMA & USB */
            #endif
			/* VA: 0xFE90_6000 */
			x = 0xc7000000 + ((x << 20) >> 20);
			return x;
		} else if ((x & 0xfe900000) == 0xfe900000) { /* SFLASH, FPC, UART0(h60), GPIO,
			                                        I2C0(h60)/EFUSE(h61), ICC, 
			                                        UART0(h60), I2C0(h60)  */
			x = 0xc0000000 + (((x >> 12) << 28) >> 4) +
			    ((x << 20) >> 20);
			return x;
		} 
		  else if ((x & 0xfe8f8000) == 0xfe8f8000) { /* SRAM */
			x -= 0xfc8f8000;
			return x;
		} 
            #if defined(CONFIG_MACH_FUJITSU_TVSTBSOC1)
		/* VA: 0xf900_0000 --- 0xf980_3FFF */
		else if ((x & 0xfff00000) == 0xf9800000) { /* nand flash (h61) */
			x = 0xe8000000 + ((x<<16)>>16);
			return x;
		}
		/* VA: 0xf900_0000 */
		else if ((x & 0xfff0f000) == 0xf9000000) { /* CEC (h61) */
			x = 0xc9000000 + ((x<<16)>>16);
			return x;
		}
		/* VA: 0xf900_1000 */
		/* VA: 0xf900_3000 */
		else if ((x & 0xfff01000) == 0xf9001000) { /* USB PHY 0/1 (h61) */
			x = 0xc7000000 + ((x<<16)>>16);
			return x;
		}
            #endif
		 else if ((x & 0xfe8f0000) == 0xfe8f0000) { /* ROM */
			x -= 0xfe8f0000;
			return x;
		} else if ((x & 0xf81c0000) == 0xf81c0000) { /* PINT */
			/* VA: 0xf81d_2000 - 0xf81f_2000 */
			x -= 0x970f0000;
			return x;
		} else if ((x & 0xf8100000) == 0xf8100000) { /* VIDDEC */
			x = 0x60000000 + ((x << 12) >> 12);
			return x;
		} else {                                     /* None */
			x -= 0x60000000;
			return x;
		}

	} else {
		/* VA: 0xe000_0000 */
		if ((x & 0xe0000000) == 0xe0000000) { /* UPI */
			x -= 0x60000000;
			return x;
		}
		/* VA: 0xd000_0000 */
		if ((x & 0xd0000000) == 0xd0000000) {
            #if (CONFIG_MEM_SIZE_RAMIFS != 0)
			if ((x & 0x8000000) == 0x8000000) { /* DDR2-V */
				x -= 0x98000000;
				return x;
            #ifndef CONFIG_MB86HXX_LARGEMEM
			} else if ((x >= MB86HXX_BUF_VA_BASE) && ((MB86HXX_BUF_VA_BASE + PHYS_SIZE ) > x)) {

				x -= (0xb0000000 + (0x8000000 - PHYS_SIZE));
				return x;
            #endif /* CONFIG_MB86HXX_LARGEMEM */
			} else {                            /* DDR2-S */
				if ((x >= MB86HXX_BUF_VA_BASE) && (x < 0xd8000000)) {
					x -= 0xb8000000 - CONFIG_FIRM_AREA_SIZE;
				}
				else {
					/* kernel(vmalloc) */
				}
				return x;
			}
            #else
			if (x < 0xd8000000) {
				if (x >= MB86HXX_BUF_VA_BASE) { /* BUF */
					x -= 0x98000000 - CONFIG_FIRM_AREA_SIZE;
				}
				else {
					/* kernel(vmalloc) */
				}
			}
			else {                               /* DDR2-V */
				x -= 0x98000000 - CONFIG_MEM_SIZE_RAMIFV + CONFIG_FIRM_FRAME_BUFFER_SIZE;
			}
            #endif /* (CONFIG_MEM_SIZE_RAMIFS != 0) */
		}
	}
	return x;
}

EXPORT_SYMBOL(io_phys_to_virt);
EXPORT_SYMBOL(io_virt_to_phys);
