/*
 *  linux/arch/arm/mach-mb86hxx/mb86h60_devkit.c
 *
 * Copyright (C) 2008-2010 FUJITSU MICROELECTRONICS LIMITED
 * Copyright (C) 2010-2011 FUJITSU SEMICONDUCTOR LIMITED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/init.h>
#include <linux/device.h>
#include <linux/sysdev.h>
#include <linux/platform_device.h>
#include <linux/amba/bus.h>
#include <linux/usb/musb.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/time.h>
#include <asm/mach/flash.h>
#include <mach/gpio.h>

#include "core.h"

#define    ETH_BASE    MB86HXX_ETH_BASE
#define    ETH_SIZE    (0x04000000)
#define    IRQ_ETHER   IRQ_ETH

static struct map_desc mb86h60_io_desc[] __initdata = {
	{
		.virtual	= MB86HXX_DDR1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DDR1_BASE),
#ifdef CONFIG_MB86HXX_LARGEMEM
		.length		= PHYS_SIZE,
#else
		.length		= 0x8000000,
#endif /* CONFIG_MB86HXX_LARGEMEM */
		.type		= MT_DEVICE
	}, {
#ifndef CONFIG_MB86HXX_LARGEMEM
		.virtual	= MB86HXX_BUF_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_BUF_BASE),
		.length		= (PHYS_SIZE - CONFIG_MEM_SIZE_RAMIFS),
		.type		= MT_DEVICE
	}, {
#endif /* CONFIG_MB86HXX_LARGEMEM */
		.virtual	= MB86HXX_DDR2_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DDR2_BASE),
		.length		= 0x8000000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_UPI_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_UPI_BASE),
		.length		= 0x18002000,
		.type		= MT_DEVICE
	}, {
#if 0
		.virtual	= MB86HXX_SFLASH_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SFLASH_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
#endif
		.virtual	= MB86HXX_VIDDEC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_VIDDEC_BASE),
		.length		= 0xD2000,
		.type		= MT_DEVICE
	}, {
	        .virtual        = MB86HXX_PINT_VA_BASE,
	        .pfn            = __phys_to_pfn(MB86HXX_PINT_BASE),
	        .length         = 0x20000,
                .type           = MT_DEVICE
	}, {
		.virtual	= MB86HXX_BOOTROM_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_BOOTROM_BASE),
		.length		= 0x5000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SRAM_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SRAM_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SFLASH_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SFLASH_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_FPCIR_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_FPCIR_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
	        .virtual	= MB86HXX_UART0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_UART0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_GPIO_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_GPIO_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_I2C0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_I2C0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_ICC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_ICC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual        = MB86HXX_USB_VA_BASE,
		.pfn            = __phys_to_pfn(MB86HXX_USB_BASE),
		.length         = 0x1000,
		.type           = MT_DEVICE
	}, {
		.virtual	= MB86HXX_DMA_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DMA_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_TSD_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_TSD_BASE),
		.length		= 0xD02000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_CLKPWR_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_CLKPWR_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_VIDOUT_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_VIDOUT_BASE),
		.length		= 0x39000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_AUD_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_AUD_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_UART1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_UART1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_TIMER0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_TIMER0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SSP0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SSP0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_I2C1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_I2C1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_I2C2_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_I2C2_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SSP1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SSP1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_EFUSE_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_EFUSE_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_PIRLO_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_PIRLO_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SCHED_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SCHED_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_RAMIFS_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_RAMIFS_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_RAMIFV_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_RAMIFV_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_ETH_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_ETH_BASE),
		.length		= 0x2000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_VIC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_VIC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}
};

#if defined(CONFIG_SERIAL_AMBA_PL011_PORT0) || defined(CONFIG_SERIAL_AMBA_PL011_PORT0_MODULE)
static struct amba_device uart0_device = {
  .dev = {
    .coherent_dma_mask = ~0,
    .init_name = "apb:uart0",
    .platform_data = NULL,
  },
  .res = {
    .start = MB86HXX_UART0_BASE,
    .end =  MB86HXX_UART0_BASE + SZ_4K -1,
    .flags = IORESOURCE_MEM,
  },
  .dma_mask = ~0,
  .irq = {IRQ_UART0, NO_IRQ},
};
#endif

#if defined(CONFIG_SERIAL_AMBA_PL011_PORT1) || defined(CONFIG_SERIAL_AMBA_PL011_PORT1_MODULE)
static struct amba_device uart1_device = {
  .dev = {
    .coherent_dma_mask = ~0,
    .init_name = "apb:uart1",
    .platform_data = NULL,
  },
  .res = {
    .start = MB86HXX_UART1_BASE,
    .end =  MB86HXX_UART1_BASE + SZ_4K -1,
    .flags = IORESOURCE_MEM,
  },
  .dma_mask = ~0,
  .irq = {IRQ_UART1, NO_IRQ},
};
#endif

#if defined(CONFIG_FMAC3H) || defined(CONFIG_FMAC3H_MODULE)
static struct resource fmac3h_resources[] = {
    [0] = {
        .start = ETH_BASE,
        .end   = ETH_BASE + ETH_SIZE - 1,
        .flags = IORESOURCE_MEM,
    },
    [1] = {
        .start = IRQ_ETHER,
	.end   = IRQ_ETHER,
	.flags = IORESOURCE_IRQ,
    },
};

static struct platform_device fmac3h_device = {
    .name          = "F_MAC3H",
    .id            = 0,
    .num_resources = ARRAY_SIZE(fmac3h_resources),
    .resource      = fmac3h_resources,
};
#endif

#if defined(CONFIG_MTD_MB86HXX) || defined(CONFIG_MTD_MB86HXX_MODULE)
#define MB86H60_PFLASH_SIZE	0x01000000

static struct flash_platform_data mb86h60_pflash_data = {
	.map_name		= "cfi_probe",
	.width			= 2,
};

static struct resource mb86h60_pflash_resource = {
	.start			= MB86HXX_UPI_BASE,
	.end			= MB86HXX_UPI_BASE + MB86H60_PFLASH_SIZE - 1,
	.flags			= IORESOURCE_MEM,
};

static struct platform_device mb86h60_pflash_device = {
        .name			= "mb86hxx-pflash",
	.id			= 0,
	.dev			= {
	.platform_data	        = &mb86h60_pflash_data,
	},
	.num_resources		= 1,
	.resource		= &mb86h60_pflash_resource,
};
#endif
#if defined(CONFIG_MTD_MB86HXX_SERIAL) || defined(CONFIG_MTD_MB86HXX_MODULE)
#define MB86H60_SFLASH_SIZE 0x1000000
static struct flash_platform_data mb86h60_sflash_data = {
	.map_name		= "cfi_probe",
	.width			= 4,
};

static struct resource mb86h60_sflash_resource = {
	.start			= MB86HXX_SFLASH_BASE,
	.end			= MB86HXX_SFLASH_BASE + MB86H60_SFLASH_SIZE - 1,
	.flags			= IORESOURCE_MEM,
};


static struct platform_device mb86h60_sflash_device = {
        .name			= "mb86hxx-sflash",
	.id			= 0,
	.dev			= {
	.platform_data	        = &mb86h60_sflash_data,
	},
	.num_resources		= 1,
	.resource		= &mb86h60_sflash_resource,
};
#endif
#if defined(CONFIG_MUSB_MB86HXX) || defined(CONFIG_MUSB_MB86HXX_MODULE)
#define MB86H60_MUSB_SIZE     SZ_4K

static struct musb_hdrc_config mb86h60_musb_config = {
	.multipoint	= 1,
	.dyn_fifo	= 0,
	.big_endian	= 0,
	.dma		= 1,
	.num_eps	= 16,
	.ram_bits	= 12,
};

static struct musb_hdrc_platform_data mb86h60_musb_data = {
        .mode           = MUSB_HOST,
        .min_power      = 25,     /* x2 = 50 mA drawn from VBUS as peripheral */
	.config		= &mb86h60_musb_config,
};

static struct resource mb86h60_musb_resource[] = {
    [0] = {
        .start = MB86HXX_USB_BASE,
        .end   = MB86HXX_USB_BASE + MB86H60_MUSB_SIZE - 1,
        .flags = IORESOURCE_MEM,
    },
    [1] = {
        .start = IRQ_USB,
        .end   = IRQ_USB,
        .flags = IORESOURCE_IRQ,
    },
    [2] = {
        .start = IRQ_DMA,
        .end   = IRQ_DMA,
        .flags = IORESOURCE_DMA,
    },
};

static u64 mb86h60_dmamask = 0xffffffffUL;
static struct platform_device mb86h60_musb_device = {
        .name           = "musb_hdrc",
        .id             = -1,
        .dev            = {
        .dma_mask       = &mb86h60_dmamask,
        .coherent_dma_mask = 0xffffffff,
        .platform_data  = &mb86h60_musb_data,
        },
        .num_resources  = ARRAY_SIZE(mb86h60_musb_resource),
        .resource       = mb86h60_musb_resource,
};
#endif /* CONFIG_MUSB_MB86HXX<_MODULE> */

static void __init mb86h60_init_machine(void)
{
#if defined(CONFIG_SERIAL_AMBA_PL011_PORT0) || defined(CONFIG_SERIAL_AMBA_PL011_PORT0_MODULE)
	mb86h6X_gpio_config_out(33, GPIO_FUNCTION_UART0_TXDATA);
	mb86h6X_gpio_config_in(32, GPIO_FUNCTION_UART0_RXDATA);
	amba_device_register(&uart0_device, &iomem_resource);
#endif
#if defined(CONFIG_SERIAL_AMBA_PL011_PORT1) || defined(CONFIG_SERIAL_AMBA_PL011_PORT1_MODULE)
	mb86h6X_gpio_config_out(34, GPIO_FUNCTION_UART1_TXDATA);
	mb86h6X_gpio_config_in(35, GPIO_FUNCTION_UART1_RXDATA);
	amba_device_register(&uart1_device, &iomem_resource);
#endif
#if defined(CONFIG_FMAC3H) || defined(CONFIG_FMAC3H_MODULE)
	platform_device_register(&fmac3h_device);
#endif
#if defined(CONFIG_MTD_MB86HXX) || defined(CONFIG_MTD_MB86HXX_MODULE)
	platform_device_register(&mb86h60_pflash_device);
#endif
#if defined(CONFIG_MTD_MB86HXX_SERIAL) || defined(CONFIG_MTD_MB86HXX_MODULE)
  platform_device_register(&mb86h60_sflash_device);
#endif
#if defined(CONFIG_MUSB_MB86HXX) || defined(CONFIG_MUSB_MB86HXX_MODULE)
	platform_device_register(&mb86h60_musb_device);
#endif

}

static struct sys_timer mb86h60_timer = {
	.init = mb86h6X_timer_init,
};

static void __init mb86h60_init_irq(void)
{
	/* Setup Interrupt Controller */
	irc_init();

	/* Setup VIC */
	vic_init((void *)VIC_VA_BASE, 0, VIC_VALID_IRQ_MASK);
}

static void __init mb86h60_map_io(void)
{
	iotable_init(mb86h60_io_desc, ARRAY_SIZE(mb86h60_io_desc));
}

MACHINE_START(FUJITSU_TVSTBSOC, "Reference Board for MB86H60 Fujitsu TV/STB SoC")
	/* Maintainer: FUJITSU MICROELECTRONICS LIMITED */
	.phys_io        = 0xd0000000,
	.io_pg_offst    = 0,
	.boot_params    = 0x0,
	.map_io         = mb86h60_map_io,
	.init_irq       = mb86h60_init_irq,
	.timer          = &mb86h60_timer,
	.init_machine   = mb86h60_init_machine,
MACHINE_END
