/*
 *  linux/arch/arm/mach-mb86hxx/mb86h61_devkit.c
 *
 * Copyright (C) 2008-2010 FUJITSU MICROELECTRONICS LIMITED
 * Copyright (C) 2010-2011 FUJITSU SEMICONDUCTOR LIMITED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
 */ 

#include <linux/init.h> 
#include <linux/device.h>
#include <linux/sysdev.h>
#include <linux/dma-mapping.h>
#include <linux/platform_device.h> 
#include <linux/mmc/host.h>
#include <linux/amba/bus.h>
#include <linux/usb/musb.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/time.h>
#include <asm/mach/flash.h>
#include <mach/gpio.h>
#include <mach/sdhci.h>
#include <linux/spi/spi.h>
#include <linux/mtd/partitions.h>
#include "core.h"

#define    ETH_BASE    MB86HXX_ETH_BASE
#define    ETH_SIZE    (0x04000000)
#define    IRQ_ETHER   IRQ_ETH
#define    SIZE_1PAGE  1024

static struct map_desc mb86h61_io_desc[] __initdata = {
	{
		.virtual	= MB86HXX_BOOTROM_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_BOOTROM_BASE),
		.length		= 0x5000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SRAM_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SRAM_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
#if (CONFIG_MEM_SIZE_RAMIFS != 0)
		.virtual	= MB86HXX_DDR1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DDR1_BASE),
 #ifdef CONFIG_MB86HXX_LARGEMEM
		.length		= CONFIG_FIRM_AREA_SIZE,
 #else
		.length		= 0x8000000,
 #endif /* CONFIG_MB86HXX_LARGEMEM */
		.type		= MT_DEVICE
	}, {
 #ifndef CONFIG_MB86HXX_LARGEMEM
		.virtual	= MB86HXX_BUF_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_BUF_BASE),
		.length		= (PHYS_SIZE - CONFIG_MEM_SIZE_RAMIFS),
		.type		= MT_DEVICE
	}, {
 #endif /* CONFIG_MB86HXX_LARGEMEM */
		.virtual	= MB86HXX_DDR2_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DDR2_BASE),
		.length		= 0x8000000,
		.type		= MT_DEVICE
	},{
#else
		.virtual	= MB86HXX_BUF_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_BUF_BASE),
		.length		= CONFIG_FIRM_AREA_SIZE,
		.type		= MT_DEVICE
	}, {
 #if (CONFIG_FIRM_FRAME_BUFFER_SIZE != 0)
		.virtual	= MB86HXX_DDR2_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DDR2_BASE),
		.length		= CONFIG_FIRM_FRAME_BUFFER_SIZE,
		.type		= MT_DEVICE
	},{
 #endif
#endif /* (CONFIG_MEM_SIZE_RAMIFS != 0) */
		.virtual	= MB86HXX_VIDDEC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_VIDDEC_BASE),
		.length		= 0xD2000,
		.type		= MT_DEVICE
	},  {
		.virtual	= MB86HXX_PINT_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_PINT_BASE),
		.length		= 0x20000,
		.type		= MT_DEVICE
	},{
		.virtual	= MB86HXX_UPI_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_UPI_BASE),
		.length		= 0x18002000,
		.type		= MT_DEVICE
	},  {
		.virtual	= MB86HXX_SFLASH_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SFLASH_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_FPCIR_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_FPCIR_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_IR_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_IR_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_GPIO_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_GPIO_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
	        .virtual	= MB86HXX_EFUSE_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_EFUSE_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_ICC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_ICC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_R2R_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_R2R_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_DMA_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_DMA_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
#if 0  //the following mapping is done by above item(DMA mapping)
	}, {
		.virtual	= MB86HXX_USB_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_USB_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
#endif
	}, {
		.virtual	= MB86HXX_USBPHY0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_USBPHY0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_USBPHY1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_USBPHY1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_TSD_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_TSD_BASE),
		.length		= 0xD02000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_CEC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_CEC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_CLKPWR_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_CLKPWR_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_VIDOUT_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_VIDOUT_BASE),
		.length		= 0x39000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_AUD_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_AUD_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_UART0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_UART0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_UART1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_UART1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_TIMER0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_TIMER0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_HDMI_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_HDMI_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_I2C0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_I2C0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_I2C1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_I2C1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_I2C2_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_I2C2_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SSP0_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SSP0_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SSP1_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SSP1_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_GRAPHENG_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_GRAPHENG_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_ADC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_ADC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_PIRLO_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_PIRLO_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SCHED_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SCHED_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_RAMIFS_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_RAMIFS_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_RAMIFV_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_RAMIFV_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_VIC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_VIC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_ETH_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_ETH_BASE),
		.length		= 0x2000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SATA_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SATA_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_HDMAC_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_HDMAC_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_NAND_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_NAND_BASE),
		.length		= 0x4000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_NANDCFG_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_NANDCFG_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	}, {
		.virtual	= MB86HXX_SDIO_VA_BASE,
		.pfn		= __phys_to_pfn(MB86HXX_SDIO_BASE),
		.length		= 0x1000,
		.type		= MT_DEVICE
	},
};

#if defined(CONFIG_SERIAL_AMBA_PL011_PORT0) || defined(CONFIG_SERIAL_AMBA_PL011_PORT0_MODULE)
static struct amba_device uart0_device = {
  .dev = {
    .coherent_dma_mask = ~0,
    .init_name = "apb:uart0",
    .platform_data = NULL,
  },
  .res = {
    .start = MB86HXX_UART0_BASE,
    .end =  MB86HXX_UART0_BASE + SZ_4K -1,
    .flags = IORESOURCE_MEM,
  },
  .dma_mask = ~0,
  .irq = {IRQ_UART0, NO_IRQ},
};
#endif

#if defined(CONFIG_SERIAL_AMBA_PL011_PORT1) || defined(CONFIG_SERIAL_AMBA_PL011_PORT1_MODULE)
static struct amba_device uart1_device = {
  .dev = {
    .coherent_dma_mask = ~0,
    .init_name = "apb:uart1",
    .platform_data = NULL,
  },
  .res = {
    .start = MB86HXX_UART1_BASE,
    .end =  MB86HXX_UART1_BASE + SZ_4K -1,
    .flags = IORESOURCE_MEM,
  },
  .dma_mask = ~0,
  .irq = {IRQ_UART0, NO_IRQ},
};
#endif

#if defined(CONFIG_FMAC3H) || defined(CONFIG_FMAC3H_MODULE)
static struct resource fmac3h_resources[] = {
    [0] = {
        .start = ETH_BASE,
        .end   = ETH_BASE + ETH_SIZE - 1,
        .flags = IORESOURCE_MEM,
    },
    [1] = {
        .start = IRQ_ETHER,
	.end   = IRQ_ETHER,
	.flags = IORESOURCE_IRQ,
    },
};

static struct platform_device fmac3h_device = {
    .name          = "F_MAC3H",
    .id            = 0,
    .num_resources = ARRAY_SIZE(fmac3h_resources),
    .resource      = fmac3h_resources,
};
#endif

#if defined(CONFIG_MTD_MB86HXX) || defined(CONFIG_MTD_MB86HXX_MODULE)
#define MB86H61_PFLASH_SIZE	0x01000000

static struct flash_platform_data mb86h61_pflash_data = {
	.map_name		= "cfi_probe",
	.width			= 2,
};

static struct resource mb86h61_pflash_resource = {
	.start			= MB86HXX_UPI_BASE,
	.end			= MB86HXX_UPI_BASE + MB86H61_PFLASH_SIZE - 1,
	.flags			= IORESOURCE_MEM,
};

static struct platform_device mb86h61_pflash_device = {
	.name			= "mb86hxx-pflash",
	.id			= 0,
	.dev			= {
	.platform_data		= &mb86h61_pflash_data,
	},
	.num_resources		= 1,
	.resource		= &mb86h61_pflash_resource,
};
#endif
#if defined(CONFIG_MTD_MB86HXX_SFLASH) || defined(CONFIG_MTD_MB86HXX_MODULE)
#define MB86H61_SFLASH_SIZE 0x1000000

static struct mtd_partition mb86hxx_sflash_parts[]=
#if 0  /* Mod by hiroshi.hasegawa@pixela.jp 2011.08.30 DT270と同じパーティション配置へ変更 */
/* FSLオリジナル */
{	
#if 0
	{
		.name = "u-boot",
		.offset = 0,
		.size =128*SIZE_1PAGE,  //512kBytes
	},
	{
		.name = "u-boot ENV",
		.offset = 128*SIZE_1PAGE,
		.size = 16*SIZE_1PAGE,
	},
	{
		.name = "Kernel",
		.offset = 144*SIZE_1PAGE,
		.size = 1024*SIZE_1PAGE,
	},
#endif
	{
		.name = "rootfs",
		.offset = 5120*SIZE_1PAGE,
		.size = 11264*SIZE_1PAGE,
	},
	
}
#else  /* Mod by hiroshi.hasegawa@pixela.jp 2011.08.30 */
/* Pixelaのパーティション設定 */
{
// #24244 slot infoの2面化. updaterの互換性を保持するため, slot0はそのままmtd2におく
	{
		.name = "libooter",
		.offset = 0,
		.size = 1024 * SIZE_1PAGE,
	},
	{
		.name = "libooter env",
		.offset = 1024 * SIZE_1PAGE,
		.size = 64 * SIZE_1PAGE,
	},
	{
		.name = "slot info0",
		.offset = (1024 + 64) * SIZE_1PAGE,
		.size = 64 * SIZE_1PAGE,		
	},
	{
		.name = "nvram0",
		.offset = (1024 + 64 + 64) * SIZE_1PAGE,
		.size = 64 * SIZE_1PAGE,
	},
	{
		.name = "nvram1",
		.offset = (1024 + 64 + 64 + 64) * SIZE_1PAGE,
		.size = 64 * SIZE_1PAGE,
	},
	{
		.name = "kernel bslot",
		.offset = (1024 + 64 + 64 + 64 + 64) * SIZE_1PAGE,
		.size = 1536 * SIZE_1PAGE,
	},
	{
		.name = "rootfs",
		.offset = (1024 + 64 + 64 + 64 + 64 + 1536) * SIZE_1PAGE,
		.size = 1024 * SIZE_1PAGE,
	},
	{
		.name = "slot0",
		.offset = (1024 + 64 + 64 + 64 + 64 + 1536 + 1024) * SIZE_1PAGE,
		.size = 6 * 1024 * SIZE_1PAGE,
	},
	{
		.name = "slot1",
		.offset = (1024 + 64 + 64 + 64 + 64 + 1536 + 1024 + 6*1024) * SIZE_1PAGE,
		.size = 6 * 1024 * SIZE_1PAGE,
	},
	{
		.name = "slot info1",
		.offset = (1024 + 64 + 64 + 64 + 64 + 1536 + 1024 + 6*1024 + 6*1024) * SIZE_1PAGE,
		.size = 64 * SIZE_1PAGE
	},
	{
		.name = "reserved",
		.offset = (1024 + 64 + 64 + 64 + 64 + 1536 + 1024 + 6*1024 + 6*1024 + 64) * SIZE_1PAGE,
		.size = 192 * SIZE_1PAGE
	}
};
#endif /* Mod by hiroshi.hasegawa@pixela.jp 2011.08.30 */

static const struct flash_platform_data mb86hxx_sflash_data =
{
#if defined(CONFIG_MB86HXX_SFLASH_MX25L12845E)
	.name = "mx25l12845e",
#endif
#if defined(CONFIG_MB86HXX_SFLASH_S25FL128P)
        .name = "s25fl128p",
#endif
#if defined(CONFIG_MB86HXX_SFLASH_W25Q128BVFG)
        .name = "w25q128bvfg",
#endif
	.parts= mb86hxx_sflash_parts,
	.nr_parts = ARRAY_SIZE(mb86hxx_sflash_parts),

};
static struct spi_board_info const mb86h61_spi_board_info[]= 
{ 
	{ /* SPI FLash*/
	   .modalias = "mb86hxx_sflash",
	   .platform_data = &mb86hxx_sflash_data,
	   .mode = SPI_MODE_0,
	   .irq = 0,
	   .max_speed_hz = 4 * 1000 * 1000, /*4MHZ*/
	   .bus_num = 0,
	   .chip_select = 0, /* device number on bus (0-based)*/
	},
};

static struct resource mb86h61_sflash_resource = {
	.start			= MB86HXX_SFLASH_BASE,
	.end			= MB86HXX_SFLASH_BASE + MB86H61_SFLASH_SIZE - 1,
	.flags			= IORESOURCE_MEM,
};


static struct platform_device mb86hxx_spi_device = {
	.name			= "mb86hxx_spi",
	.id			= 0,
	.dev			= {
	//	.platform_data		= &mb86hxx_sflash_data,
		.bus			= &spi_bus_type,
	},
	.num_resources		= 1,
	.resource		= &mb86h61_sflash_resource,
};
#endif
#if defined(CONFIG_MUSB_MB86HXX) || defined(CONFIG_MUSB_MB86HXX_MODULE)
#define MB86H61_MUSB_SIZE     SZ_4K

static struct musb_hdrc_config mb86h61_musb_config = {
	.multipoint	= 1,
	.dyn_fifo	= 0,
	.big_endian	= 0,
	.dma		= 1,
	.num_eps	= 16,
	.ram_bits	= 12,
};

static struct musb_hdrc_platform_data mb86h61_musb0_data = {
	.mode		= MUSB_HOST,
	.min_power	= 25,     /* x2 = 50 mA drawn from VBUS as peripheral */
	.config		= &mb86h61_musb_config,
};

static struct musb_hdrc_platform_data mb86h61_musb1_data = {
	.mode		= MUSB_HOST,
	.min_power	= 25,     /* x2 = 50 mA drawn from VBUS as peripheral */
	.config		= &mb86h61_musb_config,
};

static struct resource mb86h61_musb0_resource[] = {
    [0] = {
        .start = MB86HXX_USBPHY0_BASE,
        .end   = MB86HXX_USBPHY0_BASE + MB86H61_MUSB_SIZE - 1,
        .flags = IORESOURCE_MEM,
    },
    [1] = {
        .start = IRQ_USB,
        .end   = IRQ_USB,
        .flags = IORESOURCE_IRQ,
    },
    [2] = {
        .start = IRQ_DMA,
        .end   = IRQ_DMA,
        .flags = IORESOURCE_DMA,
    },
};

static struct resource mb86h61_musb1_resource[] = {
    [0] = {
        .start = MB86HXX_USBPHY1_BASE,
        .end   = MB86HXX_USBPHY1_BASE + MB86H61_MUSB_SIZE - 1,
        .flags = IORESOURCE_MEM,
    },
    [1] = {
        .start = IRQ_USB,
        .end   = IRQ_USB,
        .flags = IORESOURCE_IRQ,
    },
    [2] = {
        .start = IRQ_DMA,
        .end   = IRQ_DMA,
        .flags = IORESOURCE_DMA,
    },
};

static u64 mb86h61_dmamask = DMA_BIT_MASK(32);
static struct platform_device mb86h61_musb0_device = {
        .name           = "musb_hdrc",
        .id             = -1,
        .dev            = {
        .dma_mask       = &mb86h61_dmamask,
        .coherent_dma_mask = DMA_BIT_MASK(32),
        .platform_data  = &mb86h61_musb0_data,
        },
        .num_resources  = ARRAY_SIZE(mb86h61_musb0_resource),
        .resource       = mb86h61_musb0_resource,
};

static struct platform_device mb86h61_musb1_device = {
        .name           = "musb_hdrc1",
        .id             = -1,
        .dev            = {
        .dma_mask       = &mb86h61_dmamask,
        .coherent_dma_mask = DMA_BIT_MASK(32),
        .platform_data  = &mb86h61_musb1_data,
        },
        .num_resources  = ARRAY_SIZE(mb86h61_musb1_resource),
        .resource       = mb86h61_musb1_resource,
};
#endif /* CONFIG_MUSB_MB86HXX<_MODULE> */

#if defined(CONFIG_MMC_MB86H61_SDIO) || defined(CONFIG_MMC_MB86H61_SDIO_MODULE)

static u64 mb86h61_device_hsmmc_dmamask = 0xffffffffUL;


static struct resource mb86h61_hsmmc_resource[] = {
	[0] = {
		.start = MB86HXX_SDIO_BASE,
		.end   = MB86HXX_SDIO_BASE + MB86HXX_SZ_SDIO - 1,
		.flags = IORESOURCE_MEM,
	},
	[1] = {
		.start = IRQ_SDIO,
		.end   = IRQ_SDIO,
		.flags = IORESOURCE_IRQ,
	}
};

struct mb86h6x_sdhci_platdata mb86h61_hsmmc0_def_platdata = {
	.max_width	= 4,
	.host_caps	= (MMC_CAP_4_BIT_DATA |
			   MMC_CAP_MMC_HIGHSPEED | MMC_CAP_SD_HIGHSPEED),
};



static struct platform_device mb86h61_devices_sdio = {
	.name		= "mb86h6x-sdhci",
	.id		= -1,
	.num_resources	= ARRAY_SIZE(mb86h61_hsmmc_resource),
	.resource	= mb86h61_hsmmc_resource,
	.dev		= {
		.dma_mask		= &mb86h61_device_hsmmc_dmamask,
		.coherent_dma_mask	= 0xffffffffUL,
		.platform_data		= &mb86h61_hsmmc0_def_platdata,
	},
};
#endif

static void __init mb86h61_init_machine(void)
{
#if defined(CONFIG_SERIAL_AMBA_PL011_PORT0) || defined(CONFIG_SERIAL_AMBA_PL011_PORT0_MODULE)
	/* FMC 2/4 layer pcb */
	mb86h6X_gpio_config_out(3, GPIO_FUNCTION_UART0_TXDATA);
	mb86h6X_gpio_config_in(2, GPIO_FUNCTION_UART0_RXDATA);
	amba_device_register(&uart0_device, &iomem_resource);
#endif
#if defined(CONFIG_SERIAL_AMBA_PL011_PORT1) || defined(CONFIG_SERIAL_AMBA_PL011_PORT1_MODULE)
	/* FMC 4 layer pcb */
 #if defined(CONFIG_FUJITSU_BOARDTYPE_MB86H61_FMC_DEVKIT)
	mb86h6X_gpio_config_out(65, GPIO_FUNCTION_UART1_TXDATA);   // in FSEU board, 65 pin is used as SDIO_PIN_DATA3
 #endif
	mb86h6X_gpio_config_in(64, GPIO_FUNCTION_UART1_RXDATA);
	amba_device_register(&uart1_device, &iomem_resource);
#endif
#if defined(CONFIG_FMAC3H) || defined(CONFIG_FMAC3H_MODULE)
	platform_device_register(&fmac3h_device);
#endif
#if defined(CONFIG_MTD_MB86HXX) || defined(CONFIG_MTD_MB86HXX_MODULE)
	platform_device_register(&mb86h61_pflash_device);
#endif
#if defined(CONFIG_MTD_MB86HXX_SFLASH) || defined(CONFIG_MTD_MB86HXX_MODULE) 
	spi_register_board_info(mb86h61_spi_board_info,ARRAY_SIZE(mb86h61_spi_board_info));
	platform_device_register(&mb86hxx_spi_device);
#endif
#if defined(CONFIG_MUSB_MB86HXX) || defined(CONFIG_MUSB_MB86HXX_MODULE)
 #if defined(CONFIG_FUJITSU_BOARDTYPE_MB86H61_FMC_DEVKIT)
	mb86h6X_gpio_config_out(41, 1);
	mb86h6X_gpio_config_out(42, 1);
 #endif
	platform_device_register(&mb86h61_musb0_device);
	platform_device_register(&mb86h61_musb1_device);
#endif

#if defined(CONFIG_MMC_MB86H61_SDIO) || defined(CONFIG_MMC_MB86H61_SDIO_MODULE)
        platform_device_register(&mb86h61_devices_sdio);
#endif

}

static struct sys_timer mb86h61_timer = {
	.init = mb86h6X_timer_init,
};

static void __init mb86h61_init_irq(void)
{
	/* Setup Interrupt Controller */
	irc_init();

	/* Setup VIC */
	vic_init((void *)VIC_VA_BASE, 0, VIC_VALID_IRQ_MASK);
}

static void __init mb86h61_map_io(void)
{
	iotable_init(mb86h61_io_desc, ARRAY_SIZE(mb86h61_io_desc));
        mb86h6x_default_sdhci0();
}

MACHINE_START(FUJITSU_TVSTBSOC1, "Reference Board for MB86H61 Fujitsu TV/STB SoC")
	/* Maintainer: FUJITSU SEMICONDUCTOR LIMITED */
	.phys_io        = 0xd0000000,  //0xb0000000
	.io_pg_offst    = 0,
	.boot_params    = 0x0,
	.map_io         = mb86h61_map_io,
	.init_irq       = mb86h61_init_irq,
	.timer          = &mb86h61_timer,
	.init_machine   = mb86h61_init_machine,
MACHINE_END
