/*
 * linux/arch/arm/mach-mb86hxx/vic.c
 *
 * Copyright (C) 2008-2010 FUJITSU MICROELECTRONICS LIMITED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <linux/irq.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include <asm/irq.h>

#include "vic.h"

static void vic_mask_irq(unsigned int irq)
{

	writel(1 << irq, VIC_VA_BASE + VIC_INT_ENABLE_CLEAR);

}

static void vic_unmask_irq(unsigned int irq)
{

	writel(1 << irq, VIC_VA_BASE + VIC_INT_ENABLE);

}

static struct irq_chip vic_chip = {
	.name = "VIC",
	.ack = vic_mask_irq,
	.mask = vic_mask_irq,
	.unmask = vic_unmask_irq,
};

/*
 * vic_init - initialize a vectored interrupt controller
 * @base: iomem base address
 * @irq_start: starting interrupt number
 * @vic_sources: bitmask of interrupt sources to allow
 */
void __init vic_init(void __iomem * base, unsigned int irq_start, u32 vic_sources)
{
	unsigned int i;

	writel(0, base + VIC_INT_SELECT);
	writel(0, base + VIC_INT_ENABLE);
	writel(~0, base + VIC_INT_ENABLE_CLEAR);
	writel(0, base + VIC_ITCR);
	writel(~0, base + VIC_INT_SOFT_CLEAR);

	/* Clear current interrupt vector address. */
	writel(0, base + VIC_ADDRESS);

	for (i = 0; i < 32; i++) {
		unsigned int irq = irq_start + i;
		void __iomem *reg = base + VIC_VECT_ADDR0 + (i * 4);

		/* Set up irq vector address for all interrupts.
		 * irq vector address: 0xffff0018.
		 */
		writel(IRQ_VECTOR_ADDR, reg);

		set_irq_chip(irq, &vic_chip);
		set_irq_chip_data(irq, base);

		if (vic_sources & (1 << i)) {
			set_irq_handler(irq, handle_level_irq);
			set_irq_flags(irq, IRQF_VALID | IRQF_PROBE);
		}
	}

	/* Set Priority */
	writel(IRQ_PRIO_0, base + VIC_PRIO_0);
	writel(IRQ_PRIO_1, base + VIC_PRIO_1);
	writel(IRQ_PRIO_2, base + VIC_PRIO_2);
	writel(IRQ_PRIO_3, base + VIC_PRIO_3);
	writel(IRQ_PRIO_4, base + VIC_PRIO_4);
	writel(IRQ_PRIO_5, base + VIC_PRIO_5);
	writel(IRQ_PRIO_6, base + VIC_PRIO_6);
	writel(IRQ_PRIO_7, base + VIC_PRIO_7);
	writel(IRQ_PRIO_8, base + VIC_PRIO_8);
	writel(IRQ_PRIO_9, base + VIC_PRIO_9);
	writel(IRQ_PRIO_10, base + VIC_PRIO_10);
	writel(IRQ_PRIO_11, base + VIC_PRIO_11);
	writel(IRQ_PRIO_12, base + VIC_PRIO_12);
	writel(IRQ_PRIO_13, base + VIC_PRIO_13);
	writel(IRQ_PRIO_14, base + VIC_PRIO_14);
	writel(IRQ_PRIO_15, base + VIC_PRIO_15);
	writel(IRQ_PRIO_16, base + VIC_PRIO_16);
	writel(IRQ_PRIO_17, base + VIC_PRIO_17);
	writel(IRQ_PRIO_18, base + VIC_PRIO_18);
	writel(IRQ_PRIO_19, base + VIC_PRIO_19);
	writel(IRQ_PRIO_20, base + VIC_PRIO_20);
	writel(IRQ_PRIO_21, base + VIC_PRIO_21);
	writel(IRQ_PRIO_22, base + VIC_PRIO_22);
	writel(IRQ_PRIO_23, base + VIC_PRIO_23);
	writel(IRQ_PRIO_24, base + VIC_PRIO_24);
	writel(IRQ_PRIO_25, base + VIC_PRIO_25);
	writel(IRQ_PRIO_26, base + VIC_PRIO_26);
	writel(IRQ_PRIO_27, base + VIC_PRIO_27);
	writel(IRQ_PRIO_28, base + VIC_PRIO_28);
	writel(IRQ_PRIO_29, base + VIC_PRIO_29);
	writel(IRQ_PRIO_30, base + VIC_PRIO_30);
	writel(IRQ_PRIO_31, base + VIC_PRIO_31);

}
