/*
 * sata_fj.c - FUJITSU SATA support
 *
 * Copyright 2011-2009: Fujitsu Semiconductor , all rights reserved.
 * Copyright 2005 Red Hat, Inc.  All rights reserved.
 *
 * Originally written by MANGO zhang.
 * Please ALWAYS copy linux-ide@vger.kernel.org on emails.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/blkdev.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/dmapool.h>
#include <linux/dma-mapping.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/ata_platform.h>
#include <linux/mbus.h>
#include <linux/bitops.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi_device.h>
#include <linux/libata.h>

#include "libata.h"
#define FST_DRV_NAME "fst-sata"
#define FST_DRV_VER  "0.1"

//***********************************************************************
//***********************************************************************
//**************************************************************************
//**************************************************************************
//** Defines and Macros
//**************************************************************************
//**************************************************************************
#define      FST_SATA_PORT_INTERNAL 0x0FUL

#define  ERR_IRQ                (1 << 0)	/* shift by (2 * port #) */
#define  DONE_IRQ               (1 << 1)		/* shift by (2 * port #) */

//******************************************************************************
/*
********************************************************************************
**
** Register definitions
**
********************************************************************************
*/
#define REG_SATA_PHY_CTRL1                      IO_PHYS_TO_VIRT(0xC30005C0UL)
#define REG_SATA_PHY_CTRL2                      IO_PHYS_TO_VIRT(0xC30005CCUL)
#define REG_SATA_LINK_CDR0                      IO_PHYS_TO_VIRT(0xE4000000UL)
#define REG_SATA_LINK_CDR1_R                    IO_PHYS_TO_VIRT(0xE4000004UL)
#define REG_SATA_LINK_CDR1_W                    IO_PHYS_TO_VIRT(0xE4000004UL)
#define REG_SATA_LINK_CDR2                      IO_PHYS_TO_VIRT(0xE4000008UL)
#define REG_SATA_LINK_CDR3                      IO_PHYS_TO_VIRT(0xE400000CUL)
#define REG_SATA_LINK_CDR4                      IO_PHYS_TO_VIRT(0xE4000010UL)
#define REG_SATA_LINK_CDR5                      IO_PHYS_TO_VIRT(0xE4000014UL)
#define REG_SATA_LINK_CDR6                      IO_PHYS_TO_VIRT(0xE4000018UL)
#define REG_SATA_LINK_CDR7_W                    IO_PHYS_TO_VIRT(0xE400001CUL)
#define REG_SATA_LINK_CDR7_R                    IO_PHYS_TO_VIRT(0xE400001CUL)
#define REG_SATA_LINK_CLR0                      IO_PHYS_TO_VIRT(0xE4000020UL)
#define REG_SATA_LINK_SCR0                      IO_PHYS_TO_VIRT(0xE4000024UL)
#define REG_SATA_LINK_SCR1                      IO_PHYS_TO_VIRT(0xE4000028UL)
#define REG_SATA_LINK_SCR2                      IO_PHYS_TO_VIRT(0xE400002CUL)
#define REG_SATA_LINK_SCR3                      IO_PHYS_TO_VIRT(0xE4000030UL)
#define REG_SATA_LINK_SCR4                      IO_PHYS_TO_VIRT(0xE4000034UL)
#define REG_SATA_LINK_FPBOR                     IO_PHYS_TO_VIRT(0xE4000068UL)
#define REG_SATA_LINK_FPTCR                     IO_PHYS_TO_VIRT(0xE400006CUL)
#define REG_SATA_LINK_DMACR                     IO_PHYS_TO_VIRT(0xE4000070UL)
#define REG_SATA_LINK_DBTSR                     IO_PHYS_TO_VIRT(0xE4000074UL)
#define REG_SATA_LINK_INTPR                     IO_PHYS_TO_VIRT(0xE4000078UL)
#define REG_SATA_LINK_INTMR                     IO_PHYS_TO_VIRT(0xE400007CUL)
#define REG_SATA_LINK_ERRMR                     IO_PHYS_TO_VIRT(0xE4000080UL)
#define REG_SATA_LINK_LLCR                      IO_PHYS_TO_VIRT(0xE4000084UL)
#define REG_SATA_LINK_PHYCR                     IO_PHYS_TO_VIRT(0xE4000088UL)
#define REG_SATA_LINK_PHYSR                     IO_PHYS_TO_VIRT(0xE400008CUL)
#define REG_SATA_LINK_RXBISTPD                  IO_PHYS_TO_VIRT(0xE4000090UL)
#define REG_SATA_LINK_RXBISTD1                  IO_PHYS_TO_VIRT(0xE4000094UL)
#define REG_SATA_LINK_RXBISTD2                  IO_PHYS_TO_VIRT(0xE4000098UL)
#define REG_SATA_LINK_TXBISTPD                  IO_PHYS_TO_VIRT(0xE400009CUL)
#define REG_SATA_LINK_TXBISTD1                  IO_PHYS_TO_VIRT(0xE40000A0UL)
#define REG_SATA_LINK_TXBISTD2                  IO_PHYS_TO_VIRT(0xE40000A4UL)
#define REG_SATA_LINK_BISTCR                    IO_PHYS_TO_VIRT(0xE40000A8UL)
#define REG_SATA_LINK_BISTFCTR                  IO_PHYS_TO_VIRT(0xE40000ACUL)
#define REG_SATA_LINK_BISTSR                    IO_PHYS_TO_VIRT(0xE40000B0UL)
#define REG_SATA_LINK_BISTDECR                  IO_PHYS_TO_VIRT(0xE40000B4UL)
#define REG_SATA_LINK_TESTR                     IO_PHYS_TO_VIRT(0xE40000F4UL)
#define REG_SATA_LINK_VERSIONR                  IO_PHYS_TO_VIRT(0xE40000F8UL)
#define REG_SATA_LINK_IDR                       IO_PHYS_TO_VIRT(0xE40000FCUL)
#define REG_SATA_LINK_DMADR_R                   IO_PHYS_TO_VIRT(0xE4000400UL)
#define REG_SATA_LINK_DMADR_W                   IO_PHYS_TO_VIRT(0xE4000400UL)
#define REG_SATA_LINK_FPTAGR                    IO_PHYS_TO_VIRT(0xE4000064UL)

#define REG_HDMAC_DMACR                         IO_PHYS_TO_VIRT(0xE6000000UL)
#define REG_HDMAC_DMAC0_A                       IO_PHYS_TO_VIRT(0xE6000010UL)
#define REG_HDMAC_DMAC1_A                       IO_PHYS_TO_VIRT(0xE6000020UL)
#define REG_HDMAC_DMAC0_B                       IO_PHYS_TO_VIRT(0xE6000014UL)
#define REG_HDMAC_DMAC1_B                       IO_PHYS_TO_VIRT(0xE6000024UL)
#define REG_HDMAC_DMAC0_SA                      IO_PHYS_TO_VIRT(0xE6000018UL)
#define REG_HDMAC_DMAC1_SA                      IO_PHYS_TO_VIRT(0xE6000028UL)
#define REG_HDMAC_DMAC0_DA                      IO_PHYS_TO_VIRT(0xE600001CUL)
#define REG_HDMAC_DMAC1_DA                      IO_PHYS_TO_VIRT(0xE600002CUL)

#define MMIO_BASE                               IO_PHYS_TO_VIRT(0xE4000000UL)

#define  SATA_WAIT_STATUS_TIMEOUT               1000

#define ERR_NO 			          -0
#define ERR_TIMEOUT 	             -6
#define ERR_COMMAND_FAILED        -8
#define ERR_SECTOR_COUNT	       -16

#define MAX_SG_CT		              256
#define DMA_BOUNDARY 	           0xffffUL
#define MAX_Q_DEPTH	              32
#define SATA_STATE_ERROR	        1
#define SATA_WAIT_NO_TIMEOUT       0xFFFFFFFF

static struct scsi_host_template fst_sht = {
	ATA_BASE_SHT(FST_DRV_NAME),
	.sg_tablesize		=   MAX_SG_CT / 2,
	.dma_boundary		=   DMA_BOUNDARY,
};
#define HDMAC_TRIGGER_PERIPHERAL   0       //<!
#define HDMAC_TRIGGER_SOFTWARE     1       //<!
#define HDMAC_WIDTH_BYTE           0       //<! Transfer width 1byte=8bit
#define HDMAC_WIDTH_HALF_WORD      1       //<! Transfer width half word
#define HDMAC_WIDTH_LONG_WORD      2       //<! Transfer width word
#define HDMAC_BLOCK_MODE           0       //<! Block transfer mode
#define HDMAC_BURST_MODE           1       //<! Burst transfer mode
#define HDMAC_DEMAND_MODE          2       //<! Demand transfer mode
enum
{
	HDMAC_TRIGGER_INPUT_SOFTWARE   =  0,  //<! Trigger Input Select is Software
	HDMAC_TRIGGER_INPUT_DREQ_HIGH  = 14,  //<! DREQ High level or Positive edge
	HDMAC_TRIGGER_INPUT_DREQ_LOW   = 15,  //<! DREQ Low level or Negative edge
	HDMAC_TRIGGER_INPUT_SATA_RX    = 16,  //<! SATA RX (IDREQ[0])  High level or Positive edge
	HDMAC_TRIGGER_INPUT_SATA_TX    = 17,  //<! SATA TX (IDREQ[1])  High level or Positive edge
	HDMAC_TRIGGER_INPUT_SDIO       = 18,  //<! SDIO    (IDREQ[2])  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_03   = 19,  //<! IDREQ[3]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_04   = 20,  //<! IDREQ[4]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_05   = 21,  //<! IDREQ[5]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_06   = 22,  //<! IDREQ[6]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_07   = 23,  //<! IDREQ[7]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_08   = 24,  //<! IDREQ[8]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_09   = 25,  //<! IDREQ[9]  High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_10   = 26,  //<! IDREQ[10] High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_11   = 27,  //<! IDREQ[11] High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_12   = 28,  //<! IDREQ[12] High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_13   = 29,  //<! IDREQ[13] High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_14   = 30,  //<! IDREQ[14] High level or Positive edge
	HDMAC_TRIGGER_INPUT_IDREQ_15   = 31   //<! IDREQ[15] High level or Positive edge
} ;
enum
{
	HDMAC_BEAT_NORMAL  =  0, //<! Normal transfer method (same as single transfer method)
	HDMAC_BEAT_SINGLE  =  8, //<! Single transfer method (same as normal transfer method)
	HDMAC_BEAT_INCR    =  9, //<! Performe 1 sequential access to destination address. (Use internal 64byte fifo)
	HDMAC_BEAT_WRAP4   = 10, //<! Performe 4 sequential access to destination address. (Use internal 64byte fifo)
	HDMAC_BEAT_INCR4   = 11, //<! Performe 4 sequential access to destination address. (Use internal 64byte fifo)
	HDMAC_BEAT_WRAP8   = 12, //<! Performe 8 sequential access to destination address. (Use internal 64byte fifo)
	HDMAC_BEAT_INCR8   = 13, //<! Performe 8 sequential access to destination address. (Use internal 64byte fifo)
	HDMAC_BEAT_WRAP16  = 14, //<! Performe 8 sequential access to destination address. (Use internal 64byte fifo)
	HDMAC_BEAT_INCR16  = 15, //<! Performe 8 sequential access to destination address. (Use internal 64byte fifo)
} ;
enum
{
	HDMAC_TRANSFER_MEMCOPY     = 0,
	HDMAC_TRANSFER_MEMFILL     = 1,
	HDMAC_TRANSFER_SATA_READ   = 2,
	HDMAC_TRANSFER_SATA_WRITE  = 3,
	HDMAC_TRANSFER_SDIO_READ   = 4,
	HDMAC_TRANSFER_SDIO_WRITE  = 5,
} ;

struct hdmac_ch_cfg{
	u32                      sourceAddress;
	u32                      fixedSource;
	u32                      destinationAddress;
	u32                      fixedDestination;
	u32                      transferMode;
	u32                      transferCount;
	u32                      blockCount;
	u32                      triggerType;
	u32                      inputSelect;
	u32                      transferWidth;
	u32                      beatType;
	u32                      optTransferType;
};
struct fst_sg {
	__le32 addr;
	__le32 flags_size;
	__le32 addrs_hi;
	__le32 reserved;
};
struct host_priv {

	struct dma_pool		*sg_tbl_pool;
};
struct ports_priv {

	dma_addr_t		sg_tbl_dma[1];
	struct fst_sg  * sg_tbl[1];
};

static bool fst_qc_fill_rft(struct ata_queued_cmd *qc);
static u32 soft_reset( u32 port, u32 msecs_timeout );
static u32 fst_qc_issue(struct ata_queued_cmd *qc);
static u32 fst_read_id(struct ata_device *dev,struct ata_taskfile *tf,u16*id);
static int cdr_chk_rdy( u32 timeout);
static int cdr_chk_datarq( u32 timeout );
static int cdr_chk_bsy( u32 msecsTimeOut );
static int fst_cable_detect(struct ata_port*ap);
static int fst_qc_defer(struct ata_queued_cmd *qc);
static int fst_hardreset(struct ata_link *link, u32 *class,unsigned long deadline);
static int fst_scr_read(struct ata_link *link, u32 sc_reg_in, u32 *val);
static int fst_scr_write(struct ata_link *link, u32 sc_reg_in, u32 val);
static int fst_port_start(struct ata_port *ap);
static int fst_enable_pm(struct ata_port*ap,enum link_pm policy);
static int fst_port_suspend(struct ata_port*ap,pm_message_t mesg);
static int fst_port_resume(struct ata_port*ap);
static int fst_prereset(struct ata_link *link, unsigned long deadline);
static int fst_softreset(struct ata_link *link, u32 *classes,unsigned long deadline);
static int fst_hardreset(struct ata_link *link, u32 *classes,unsigned long deadline);
static int fst_pmp_prereset(struct ata_link *link, unsigned long deadline);;
static int fst_pmp_softreset(struct ata_link *link, u32 *classes,unsigned long deadline);
static int fst_pmp_hardreset(struct ata_link *link, u32 *classes,unsigned long deadline);
static int fst_init_host(struct ata_host *host);
static int fst_ata_do_set_mode(struct ata_link *link, struct ata_device **r_failed_dev);
static void dump_header(u16* buffer);
static void fst_qc_prep(struct ata_queued_cmd *qc);
static void fst_eh_thaw(struct ata_port *ap);
static void fst_eh_freeze(struct ata_port *ap);
static void fst_port_stop(struct ata_port *ap);
static void fst_phy_reset(struct ata_port *ap);
static void fst_set_piomode(struct ata_port*ap,struct ata_device*dev);
static void fst_set_dmamode(struct ata_port*ap,struct ata_device*dev);
static void fst_set_mode(struct ata_link*link,struct ata_device**r_failed_dev);
static void fst_dev_config (struct ata_device *dev);
static void fst_pmp_attach(struct ata_port*ap);
static void fst_pmp_detach(struct ata_port*ap);
static void fst_disable_pm(struct ata_port*ap);
static void fst_host_stop(struct ata_host *host);
static void fst_postreset(struct ata_link *link, u32 *classes);
static void fst_pmp_postreset(struct ata_link *link, u32 *classes);
static void fst_post_internal_cmd(struct ata_queued_cmd *qc);
static void fst_error_handler(struct ata_port *ap);
static void fst_lost_intr(struct ata_port *ap);
static void fst_port_init(struct ata_ioports *port,  void __iomem *port_mmio);
static void fst_sff_dev_select(struct ata_port *ap, u32 device);
static void fst_bmdma_setup(struct ata_queued_cmd *qc);
static void fst_bmdma_start(struct ata_queued_cmd *qc);
static void fst_bmdma_stop(struct ata_queued_cmd *qc);
static u8 fst_sff_check_status(struct ata_port *ap);
static u8 fst_sff_check_altstatus(struct ata_port *ap);
static u8 fst_bmdma_status(struct ata_port *ap);
static s32 send_cmd( u32 port,u32 msecsTimeOut,uint8_t command,uint64_t lba,
		u32 sectors,uint8_t feature );
static void hdmac_init(void);
static void hdmac_postenable(struct ata_queued_cmd *qc);
static void hdmac_link_cfg(int rw);
static void hdmac_cfg(const u32 index,const  struct hdmac_ch_cfg config);
static int  hdmac_stop(int index);
static void hdmac_start(struct ata_queued_cmd *qc);
static void fst_sff_irq_clear(struct ata_port *ap);
static unsigned long fst_mode_filter(struct ata_device*dev,unsigned long xfer_mask);
static struct ata_port_operations fst_ops = {

	.inherits	    =  &ata_sff_port_ops,
	.qc_defer		= fst_qc_defer,
	.qc_prep		= fst_qc_prep,
	.qc_issue		= fst_qc_issue,

	.cable_detect = fst_cable_detect,
	.mode_filter  = fst_mode_filter,
	.set_piomode  = fst_set_piomode,
	.set_dmamode  = fst_set_dmamode,

	.freeze = fst_eh_freeze,
	.thaw   = fst_eh_thaw,
	.hardreset = fst_hardreset,
	.error_handler		= ata_std_error_handler, /* avoid SFF EH */
	.post_internal_cmd	= fst_post_internal_cmd,

	.scr_read  = fst_scr_read,
	.scr_write = fst_scr_write,

	.port_start = fst_port_start,
	.host_stop 	= fst_host_stop,

	.dev_config    = fst_dev_config,
	.pmp_softreset = fst_pmp_softreset,
	.pmp_hardreset = fst_pmp_hardreset,
	.softreset     = NULL,
	.sff_check_status	= fst_sff_check_status,
	.sff_irq_clear      = fst_sff_irq_clear,
	.bmdma_setup=fst_bmdma_setup,
	.bmdma_start=fst_bmdma_start,
	.bmdma_stop =fst_bmdma_stop,
	.bmdma_status=fst_bmdma_status,

	.prereset  = fst_prereset,
	.softreset = fst_softreset,
	.postreset = fst_postreset,

	.phy_reset = fst_phy_reset,
	.sff_dev_select		= fst_sff_dev_select,

};
static void fst_sff_irq_clear(struct ata_port *ap)
{
/*	int index=0,val,val1;
	val=readl(REG_HDMAC_DMAC0_B+index*0x10);
	val1 =val&0xfff8ffff;
	writel(val1,REG_HDMAC_DMAC0_B+index*0x10);
	DPRINTK("fst_sff_irq_clear, index=0 read=0x%X\n",val);
	val=readl(REG_SATA_LINK_DMACR);
	DPRINTK("fst_sff_irq_clear, DMACR=0x%X\n",val);
	writel(0,REG_SATA_LINK_DMACR);*/
}
static void fst_bmdma_setup(struct ata_queued_cmd *qc)
{
	DPRINTK("ENTER fst_bmdma_setup\n");
}
static void fst_bmdma_start(struct ata_queued_cmd *qc)
{
	DPRINTK("ENTER fst_bmdma_start\n");
}
static void fst_bmdma_stop(struct ata_queued_cmd *qc)
{
	DPRINTK("ENTER fst_bmdma_stop\n");
}
static u8 fst_bmdma_status(struct ata_port *ap)
{
	DPRINTK("ENTER fst_bmdma_status\n");
		return 0;
}
static int fst_ata_do_set_mode(struct ata_link *link, struct ata_device **r_failed_dev)
{
/*	struct ata_port *ap = link->ap;
	struct ata_device *dev;
	int rc = 0, used_dma = 0, found = 0;

 out:
	if (rc)
			*r_failed_dev = dev;
		return rc;*/
	return 0;
}
static u8 fst_sff_check_altstatus(struct ata_port *ap)
{
	u8 val=readb(ap->ioaddr.ctl_addr);
	return val;
}
static void fst_lost_intr(struct ata_port *ap)
{
	DPRINTK("fst_lost_interrupt, nothing \n");
}
static void fst_error_handler(struct ata_port *ap)
{
	DPRINTK("fst_error_handler, nothing \n");
}
static void fst_post_internal_cmd(struct ata_queued_cmd *qc)
{
	/*
	!!MNG according to libata-core.c, ata_exec_internal_sg, here we should deal with the qc->flags
	*/
	//if(qc->tf.command==0x27)qc->flags &=~ATA_QCFLAG_FAILED;
	DPRINTK("fst_post_inter.._cmd, cmd:0x%X, qc->flags=0x%X\n",qc->tf.command,(int)qc->flags);

}
static bool fst_qc_fill_rft(struct ata_queued_cmd *qc)
{
	DPRINTK("fst_qc_fill_rft\n");
	return 0;
}
static int fst_prereset(struct ata_link *link, unsigned long deadline)
{
	struct ata_eh_context *ehc = &link->eh_context;
	DPRINTK("fst_prereset i.action=0x%X\n",ehc->i.action);
	if(ehc->i.action&ATA_EH_RESET==ATA_EH_RESET)
	{
		//ehc->i.action &=~ATA_EH_RESET;
		DPRINTK("fst_prereset (nothing) ~ATA_EH_RESET\n");
	}

	return 0;
}
static int fst_softreset(struct ata_link *link, u32 *classes,unsigned long deadline)
{
	DPRINTK("fst_softreset\n");
	return 0;

}
static int fst_hardreset(struct ata_link *link, u32 *classes,unsigned long deadline)
{
	DPRINTK("fst_hardreset\n");
	return 0;
}
static void	fst_postreset(struct ata_link *link, u32 *classes)
{
	u8 val;

	classes[0]=ATA_DEV_ATA;
	classes[1]=ATA_DEV_ATA;

	/* reset complete, clear SError */
	val=readb(REG_SATA_LINK_CDR1_R);
	writeb(val,REG_SATA_LINK_CDR1_R);
	DPRINTK("fst_postreset cls[0]=0x%X,cls[1]=0x%X\n",classes[0],classes[1]);

}

static void fst_set_piomode(struct ata_port*ap,struct ata_device*dev)
{
	s32 port,timeout,command,lba,sectors,feature,ret;

	timeout=100;
	port = ap->port_no;
	dev->flags=ATA_DFLAG_PIO;

	DPRINTK("fst_set_piomode is iordy,is iordy >> send_cmd\n");
	//step 1.1  Set default PIO Mode with IORDY
	command = ATA_CMD_SET_FEATURES;//SATA_COMMAND_SET_FEATURE;
	lba     = 0;
	sectors = XFER_PIO_SLOW; //=0
	feature = SETFEATURES_XFER;  //feature sub-command
	ret=send_cmd(port,timeout,command,lba,sectors,feature);

	command = ATA_CMD_SET_FEATURES;
	lba     = 0;
	sectors = (XFER_PIO_0 ); //=8
	feature = SETFEATURES_XFER;
	ret=send_cmd(port,timeout,command,lba,sectors,feature);

	command = ATA_CMD_SET_MULTI;
	lba     = 0;
	sectors = (XFER_MW_DMA_0 );
	feature = 0;
	ret=send_cmd(port,timeout,command,lba,sectors,feature);

}
static void fst_set_dmamode(struct ata_port*ap,struct ata_device*dev)
{
	s32 port,timeout,command,lba,sectors,feature,ret;
	timeout=100;
/*	!!MNG, set multi DMA mode*/
	port = ap->port_no;
	command = ATA_CMD_SET_FEATURES;
	lba     = 0;
	sectors = XFER_MW_DMA_2;
	feature = SETFEATURES_XFER;
	ret=send_cmd(port,timeout,command,lba,sectors,feature);
	//step 4.
	// Set multiple read/write sectors
	command = ATA_CMD_SET_MULTI;
	lba     = 0;
	sectors = XFER_MW_DMA_0;//id[47]&0xff//256; !!MNG,if sectors=32,then command error ,why
	feature = 0;
	//ret=send_cmd(port,timeout,command,lba,sectors,feature);
	DPRINTK("fst_set_dmamod >> nothing\n");
}
static void fst_set_mode(struct ata_link*link,struct ata_device**r_failed_dev)
{
	DPRINTK("fst_set_mode\n");
}
static u32 fst_read_id(struct ata_device *dev,struct ata_taskfile *tf,u16*id)
{
	u32 err_mask = ata_do_dev_read_id(dev, tf, id);
	DPRINTK("fst_read_id >> send_cmd err_mask=0x%X\n",err_mask);
	return err_mask;
}
static void fst_dev_config (struct ata_device *adev)
{

/*
	suport LBA
		LBA48
		NCQ
		FLUSH_EXT
		HIPM
		PIO*/
	adev->flags|=ATA_DFLAG_PIO;// using PIO Mode
/*
	adev->multi_count=1;
	adev->pio_mask  =0x1f;
	adev->mwdma_mask=0x07;
	adev->udma_mask =0x7f;*/
	DPRINTK("fst_dev_config,dev->flags=0x%x\n",(int)adev->flags);
}

static void fst_phy_reset(struct ata_port *ap)
{
	s32 lba,sectors,feature,port,timeout,command,ret ;

	port = ap->port_no;
	timeout = 100;
	command = ATA_CMD_DEV_RESET;
	lba 	= 1;
	sectors = 0;
	feature = 0;
	ret=send_cmd(port,timeout,command,lba,sectors,feature);
	DPRINTK("ATA_CMD_DEV_RESET\n ");
}

static u32 fst_qc_issue(struct ata_queued_cmd *qc)
{
   struct ata_port *ap = qc->ap;
	int port_irqs;
	DPRINTK("fst_qc_issue,prot=0x%X (3=DMA,4=NCQ),cmd=0x%X\n",qc->tf.protocol,qc->tf.command);
	switch (qc->tf.protocol) {
	case ATA_PROT_DMA:
	case ATA_PROT_NCQ:
		/*special dealing*/
	case ATA_PROT_PIO:
		/*
		 * Errata SATA#16, SATA#24: warn if multiple DRQs expected.
		 *
		 * Someday, we might implement special polling workarounds
		 * for these, but it all seems rather unnecessary since we
		 * normally use only DMA for commands which transfer more
		 * than a single block of data.
		 *
		 * Much of the time, this could just work regardless.
		 * So for now, just log the incident, and allow the attempt.
		 */
		/* drop through */
	case ATA_PROT_NODATA:
	case ATAPI_PROT_PIO:
	case ATAPI_PROT_NODATA:
		if (ap->flags & ATA_FLAG_PIO_POLLING)
			qc->tf.flags |= ATA_TFLAG_POLLING;
		break;
	}

	if (qc->tf.flags & ATA_TFLAG_POLLING)
		port_irqs = ERR_IRQ;	        /* mask device interrupt when polling */
	else
		port_irqs = ERR_IRQ | DONE_IRQ;	/* unmask all interrupts */

	if (qc->tf.command == ATA_CMD_READ_LOG_EXT) {
		DPRINTK("fst_qc_issue send FIS\n");
	}

	return ata_sff_qc_issue(qc);
}

static u8 fst_sff_check_status(struct ata_port *ap)
{
	u8 stat = ioread8(ap->ioaddr.status_addr);
/*	u32 err=0;
	if(stat&1)
	{
		err=readl(REG_SATA_LINK_ERRMR);
	}
	DPRINTK("[CDR7_R=0x%X,LINK_ERRMR=0x%X] ",stat,err);*/
	DPRINTK("|_|");
	return stat;
}

static void fst_qc_prep(struct ata_queued_cmd *qc)
{
	int si;
	struct scatterlist *sg;
	struct ata_taskfile *tf = &qc->tf;

	DPRINTK("fst_qc_prep qc->tag:0x%X,qc->flags:0x%X,cmd:0x%X\n",qc->tag,(int)qc->flags,tf->command);

	for_each_sg(qc->sg, sg, qc->n_elem, si) {
		dma_addr_t addr = sg_dma_address(sg);
		u32 sg_len = sg_dma_len(sg);

		while (sg_len) {
			u32 offset = addr & 0xffff;
			u32 len = sg_len;

			if (offset + len > 0x10000)
				len = 0x10000 - offset;
			sg_len -= len;
			addr += len;
			DPRINTK("fst_qc_prep iterate sg: offset:0x%X,len:0x%d\n",offset,len);
		}
	}

}
static int fst_qc_defer(struct ata_queued_cmd *qc)
{
	struct ata_port *ap = qc->ap;
	DPRINTK("fst_qc_defer nr_active_links=0x%d\n",ap->nr_active_links);
	return 0;
}

static 	int fst_pmp_softreset(struct ata_link *link, u32 *classes,unsigned long deadline)
{
	DPRINTK("fst_pmp_softreset\n");
	return 0;
}
static 	int	fst_pmp_hardreset(struct ata_link *link, u32 *classes,unsigned long deadline)
{
	DPRINTK("fst_pmp_hardreset\n");
	return 0;
}
static int fst_cable_detect(struct ata_port*ap)
{
	DPRINTK("fst_cable_detect\n");
	return ATA_CBL_SATA;
}
static void fst_host_stop(struct ata_host *host)
{
	DPRINTK("fst_host_stop\n");
}
static unsigned long fst_mode_filter(struct ata_device*dev,unsigned long xfer_mask)
{
	DPRINTK("fst_mode_filter\n");
	return 0;
}
/*
static 	int	fst_pmp_prereset(struct ata_link *link, unsigned long deadline)
{
	DPRINTK("fst_pmp_prereset\n");
	return 0;
}
static 	void	fst_pmp_postreset(struct ata_link *link, u32 *classes)
{
	DPRINTK("fst_pmp_postreset\n");
}
static void fst_pmp_attach(struct ata_port*ap)
{
	DPRINTK("fst_pmp_attach\n");
}
static void fst_pmp_detach(struct ata_port*ap)
{
	DPRINTK("fst_pmp_detach\n");
}
static int fst_enable_pm(struct ata_port*ap,enum link_pm policy)
{
	DPRINTK("fst_enable_pm\n");
	return 0;
}
static void fst_disable_pm(struct ata_port*ap)
{
	DPRINTK("fst_disable_pm\n");
}
*/

static int fst_port_suspend(struct ata_port*ap,pm_message_t mesg)
{
	DPRINTK("FJ fst_port_suspend\n");
	return 0;
}
static int fst_port_resume(struct ata_port*ap)
{
	DPRINTK("fst_port_resume\n");
	return 0;
}
static void fst_eh_thaw(struct ata_port *ap)
{
	DPRINTK("eh_thaw\n");
}
static void fst_eh_freeze(struct ata_port *ap)
{
/*	int index=0;
	hdmac_stop(index);
	enable_port_irqs(ap, 0);*/
	DPRINTK("fst_eh_freeze \n");
}
static int fst_scr_read(struct ata_link *link, u32 sc_reg_in, u32 *val)
{

	s32 base = REG_SATA_LINK_SCR0;
	*val = readl(sc_reg_in * 4 + base);
	DPRINTK("\tscr_read : sc_reg_in=%d val = 0x%x port=%d\n",sc_reg_in,*val,link->ap->port_no);
	return 0;
}
static int fst_scr_write(struct ata_link *link, u32 sc_reg_in, u32 val)
{
	s32 base = REG_SATA_LINK_SCR0;
	writel(val,sc_reg_in * 4 + base);
	DPRINTK("\tscr_write : sc_reg_in=%d,val=0x%X\n",sc_reg_in,val);
	return 0;
}

static int fst_port_start(struct ata_port *ap)
{
/*	struct ports_priv *pp=ap->private_data;
	struct host_priv *hp=ap->host->private_data;
	pp->sg_tbl[0]=dma_pool_alloc(hp->sg_tbl_pool,GFP_KERNEL, &pp->sg_tbl_dma[0]);*/
	DPRINTK("ENTER  port_start\n");
	#define ATA_DEBUG
	ap->print_id=ap->port_no;

	fst_phy_reset(ap);
	msleep(100);
	return 0;
}


static void fst_port_stop(struct ata_port *ap)
{
	DPRINTK("fst_port_stop()\n");
}
static const struct ata_port_info fst_port_info[] = {
	{  	/* 0 */
		.flags		=  ATA_FLAG_SATA | ATA_FLAG_PMP ,
		.pio_mask	= ATA_PIO4,
		//.udma_mask	= ATA_MWDMA4,
		//.mwdma_mask = ATA_MWDMA4,
		.port_ops	= &fst_ops,
	},
	{  	/* 0 */
		.flags		=  ATA_FLAG_SATA |ATA_FLAG_PMP ,
		.pio_mask	= ATA_PIO4,
		//.udma_mask	= ATA_MWDMA4,
		//.mwdma_mask = ATA_MWDMA4,
		.port_ops	= &fst_ops,
	},
	{  	/* 0 */
		.flags		=  ATA_FLAG_SATA |ATA_FLAG_PMP ,
		.pio_mask	= ATA_PIO4,
		//.udma_mask	= ATA_MWDMA4,
		//.mwdma_mask = ATA_MWDMA4,
		.port_ops	= &fst_ops,
	},

};

static s32 send_cmd(u32 port,u32 mt,uint8_t  cmd,uint64_t lba,
				   u32 sectors,uint8_t  fea )
{
	u32 device_mask   = 0x40;
	s32  result       = ERR_NO;
	s32  val;
	// Set Port Multiplier Port
	val = readl(REG_SATA_LINK_SCR2);
	val |= port<<16;
	writel(val,REG_SATA_LINK_SCR2);
	writel(fea,REG_SATA_LINK_CDR1_W);
	switch(cmd)
	{
		case ATA_CMD_DEV_RESET:
		case ATA_CMD_SET_FEATURES:
		case ATA_CMD_SET_MULTI:
			writel(((uint8_t) (lba>>24) & 0x0F) | (device_mask & 0xF0),REG_SATA_LINK_CDR6);
			writel((uint8_t) (lba>>16) & 0xFF ,	REG_SATA_LINK_CDR5);
			writel((uint8_t) (lba>>8)  & 0xFF ,	REG_SATA_LINK_CDR4);
			writel((uint8_t)  lba      & 0xFF,	REG_SATA_LINK_CDR3);
			writel(sectors,	REG_SATA_LINK_CDR2);
			result = cdr_chk_bsy( mt );

			if( result != ERR_NO )
			{
				DPRINTK("CMD Device is busy!\n");
				return ERR_COMMAND_FAILED;
			}

			result = cdr_chk_datarq( mt );
			if( result != ERR_NO )
			{
				DPRINTK("CMD Device NOT ready for transfer!\n");
				return ERR_COMMAND_FAILED;
			}
			writel(cmd,REG_SATA_LINK_CDR7_W);
			result = cdr_chk_rdy( mt );
			if( result != ERR_NO )
			{
				DPRINTK("CMD Device not ready!\n");
				return ERR_COMMAND_FAILED;
			}
			val = readl(REG_SATA_LINK_CDR7_R);
			if( val& SATA_STATE_ERROR )
			{
				return ERR_COMMAND_FAILED;
			}
			break;
		default:
			DPRINTK("Do not support \n");
			return ERR_COMMAND_FAILED;
	}
	return result;
}

u32 fst_interrupt(int irq, void *dev_instance)
{
	u32 irq_status_reg = readl(REG_SATA_LINK_INTPR);//interrupt pending register
	u32 notify_reg     = readl(REG_SATA_LINK_SCR4);
	u32 serror_reg     = readl(REG_SATA_LINK_SCR1);
	struct ata_host *host = dev_instance;
	struct ata_port *ap = host->ports[0];
	struct ata_queued_cmd *qc;
	int val=0,i;

	// Clear IRQ's
	writel(irq_status_reg,REG_SATA_LINK_INTPR);

	// Clear Notify
	writel(notify_reg,REG_SATA_LINK_SCR4);

	// Clear Serror
	writel(serror_reg,REG_SATA_LINK_SCR1);


	for(i=0;i<host->n_ports;i++)
	{
		ap=host->ports[i];
		/* !!MNG, a FIS recevied from the device.*/
		if( irq_status_reg&0x80000000 )
		{
			val=readl(REG_SATA_LINK_CDR7_R);
			/*val=readl(REG_HDMAC_DMAC0_B+index*0x10);
			DPRINTK( " serror    (DMAC0_B   ): 0x%08x\n", val);*/
			spin_lock(&host->lock);
			qc= ata_qc_from_tag(ap, ap->link.active_tag);
			if (qc) {
				if (qc->tf.flags & ATA_TFLAG_POLLING)
					qc = NULL;
				else if (!(qc->flags & ATA_QCFLAG_ACTIVE))
					qc = NULL;
			}
			if (qc)
			{
				DPRINTK("INTR >> ata_sff_host_intr \n");
				ata_sff_host_intr(ap, qc);
			}
			spin_unlock(&host->lock);
		}
	}
	DPRINTK( " Isr status(LINK_INTPR): 0x%08x CDR7_R=0x%08x\n", irq_status_reg,val);
	DPRINTK( " Isr notify(LINK_SCR4 ): 0x%08x\n", notify_reg);
	DPRINTK( " serror    (LINK_SCR1 ): 0x%08x\n", serror_reg);
	return(1);
}

static s32 cdr_chk_rdy( u32 timeout)
{
	u32 start_time;
	u32 check_time;
	uint8_t val,drdy,bsy;

	start_time = jiffies;

	val = readb(REG_SATA_LINK_CDR7_R);
	drdy=(val&0x40)>>6;
	bsy =(val&0x80)>>7;
	if((bsy==0)&&(drdy==1))
	{
		return( ERR_NO );
	}

	for(;;)
	{
		val = readb(REG_SATA_LINK_CDR7_R);
		drdy= (val&0x40)>>6;
		bsy = (val&0x80)>>7;
		if((bsy==0)&&(drdy==1))
		{
			return( ERR_NO );
		}
		if( timeout != SATA_WAIT_NO_TIMEOUT )
		{
			check_time = jiffies;
			if( ( check_time - start_time ) > timeout )
			{
				DPRINTK("\tcdr_chk_rdy CDR7_R=0x%X\n",val);
				return( ERR_TIMEOUT );
			}
		}
	}
}
static s32 cdr_chk_datarq( u32 timeout )
{
	u32 start_time;
	u32 check_time;
/*	!!MNG, please do not use u32*/
	uint8_t val,drq;

	start_time = jiffies;
	val= readb(REG_SATA_LINK_CDR7_R);

	drq= val&0x08;
	if(drq == 0x00)
	{
		return( ERR_NO );
	}

	for(;;)
	{
		val = readb(REG_SATA_LINK_CDR7_R);
		drq = val&0x08;
		if(drq == 0x00)
		{
			return( ERR_NO );
		}
		check_time = jiffies;
		if( ( check_time - start_time ) > timeout )
		{
			DPRINTK("\tcdr_chk_datarq really val=%x\n",val);
			return( ERR_TIMEOUT );
		}
	}

}

static s32 cdr_chk_bsy( u32 msecsTimeOut )
{
	u32 start_time;
	u32 check_time;
	uint8_t val;

	start_time = jiffies;

	val =readb(REG_SATA_LINK_CDR7_R);
	if((val&0x80) == 0 )
	{
		return( ERR_NO );
	}

	for(;;)
	{
		val=readb(REG_SATA_LINK_CDR7_R);
		if( (val&0x80) == 0 )
		{
			return( ERR_NO );
		}
		check_time = jiffies;
		if( ( check_time - start_time ) > msecsTimeOut )
		{
			DPRINTK("\tcdr_chk_bsy\n");
			return( ERR_TIMEOUT );
		}
	}

}
static u32 soft_reset( u32 port, u32 msecs_timeout )
{

	s32 result     = ERR_NO;
	u32 signature = 0;
	s32 val;
	// Set PM Port
	val = readl(REG_SATA_LINK_SCR2);
	val |= port<<16;
	writel(val,REG_SATA_LINK_SCR2);

	// Software reset to obtain device signature
	writel(0x04,REG_SATA_LINK_CLR0);
	msleep(10);
	writel(0x00,REG_SATA_LINK_CLR0);

	result = cdr_chk_bsy(msecs_timeout);
	if( result != ERR_NO )
		return(0);

	//Read low order bytes (HOB=0)
	val = readl(REG_SATA_LINK_CLR0);
	val &= 0x7f;
	writel(val,REG_SATA_LINK_CLR0);

	signature   = ( readl(REG_SATA_LINK_CDR2) & 0xFF )  <<  0;
	signature |= ( readl(REG_SATA_LINK_CDR3) & 0xFF )  <<  8;
	signature |= ( readl(REG_SATA_LINK_CDR4) & 0xFF )  << 16;
	signature |= ( readl(REG_SATA_LINK_CDR5) & 0xFF )  << 24;

	return(signature);

}
static int scr_chk_det( u32 msecs_timeout )
{
    u32 start_time;
    u32 check_time;
    u32 state;

    start_time = jiffies;

    state = readl(REG_SATA_LINK_SCR0);
    state &= 0xf;
    //PHY READ signal detected.
    if( state == 0x3 )
    {
        return( ERR_NO );
    }

    for(;;)
    {
    	state = readl(REG_SATA_LINK_SCR0);
    	state &= 0xf;
        DPRINTK("\tcsr_chk_det =%d\n",state);
        if( state == 0x3 )
        {
            return( ERR_NO );
        }
        check_time = jiffies;
        if( ( check_time - start_time ) > msecs_timeout )
        {
            return( ERR_TIMEOUT );
        }
    }
}

int phy_init(void)
{
	u32 value;
	s32 result;

	for(;;)
	{
		//power on reset
		writel(0x00096461,REG_SATA_PHY_CTRL1);
		value = readl(REG_SATA_PHY_CTRL1);
		value &=0xfffffffe;
		writel(value,REG_SATA_PHY_CTRL1);

		value = readl(REG_SATA_PHY_CTRL1);
		value |=0x1;
		writel(value,REG_SATA_PHY_CTRL1);

		//disable all interrrupt
		writel(0,REG_SATA_LINK_INTMR);
		/* default value is 0, but see page 785, TXOPCNT, TXODCNT */
		writel(0x00000D20,REG_SATA_LINK_PHYCR);

		result = scr_chk_det(SATA_WAIT_STATUS_TIMEOUT);
		if( result != ERR_NO )
			break;

		result = soft_reset( (int)FST_SATA_PORT_INTERNAL, 0);
		DPRINTK("sig=0x%X\n",result);
		break;
		//TXOPCNT TXODCNT
	}
	return result;
}

static void dump_header(u16* buffer)
{
	DPRINTK("removeable: %d\n",(buffer[0]&0x80 == 0x80)?1:0);
	DPRINTK("cylinders : %d\n", 	buffer[1]);
	DPRINTK("headers   : %d\n", 	buffer[3]);
	DPRINTK("sectors   : %d\n", 	buffer[6]);
	DPRINTK("maxmulsec : %d\n", 	buffer[47]&0xff);
	DPRINTK("piomodenum: %d\n",	buffer[51]>>8);
}
static void fst_sff_dev_select(struct ata_port *ap, u32 device)
{
	u8 dev;

	if (device == 0)
		dev = ATA_DEVICE_OBS;
	else
		dev = ATA_DEVICE_OBS | ATA_DEV1;

	/*!!MNG,why to do this?
	!!MNG,it will write CDR6*/
	writeb(dev,ap->ioaddr.device_addr);
	DPRINTK("fst_sff_dev_select,device=0x%X.\n",dev);
	ata_sff_pause(ap);	/* needed; also flushes, for mmio */
}
static void fst_port_init(struct ata_ioports *port,  void __iomem *port_mmio)
{
	void __iomem *shd_base = port_mmio;
	/* PIO related setup
	 */
	port->data_addr 	= shd_base + (sizeof(u32) * ATA_REG_DATA);
	port->error_addr 	=
					port->feature_addr = shd_base + (sizeof(u32) * ATA_REG_ERR);
	port->nsect_addr 	= shd_base + (sizeof(u32) * ATA_REG_NSECT);
	port->lbal_addr 	= shd_base + (sizeof(u32) * ATA_REG_LBAL);
	port->lbam_addr   = shd_base + (sizeof(u32) * ATA_REG_LBAM);
	port->lbah_addr 	= shd_base + (sizeof(u32) * ATA_REG_LBAH);
	port->device_addr = shd_base + (sizeof(u32) * ATA_REG_DEVICE);
	port->status_addr =
							port->command_addr = shd_base + (sizeof(u32) * ATA_REG_STATUS);
	/* special case: control/altstatus doesn't have ATA_REG_ address */
	port->altstatus_addr = port->ctl_addr = REG_SATA_LINK_CLR0;//REG_SATA_PHY_CTRL1;
	/* unused: */
	port->cmd_addr = port->bmdma_addr = port->scr_addr = NULL;
}
static int fst_init_host(struct ata_host *host)
{
	int port;
	struct ports_priv *pp;
	for (port = 0; port < host->n_ports; port++) {

		struct ata_port *ap = host->ports[port];
		pp=devm_kzalloc(ap->host->dev,sizeof(*pp),GFP_KERNEL);
		ap->private_data=pp;

			fst_port_init(&ap->ioaddr, (void*)MMIO_BASE);
	}
	DPRINTK("sizeof dma_addr_t=%d\n",sizeof(dma_addr_t));
	return 0;
}

static void hdmac_init(void)
{
	int val,index;
	/*!!MNG, hdmac init*/
	for(index=0; index<2; index++)
	{
		// TODO: Added channel config default settings
		writel(0,index*0x10+REG_HDMAC_DMAC0_A);
		// Clears all IRQ's
		/*!!MNG, ss=0,18:16*/
		val=readl(index*0x10+REG_HDMAC_DMAC0_B);
		val &=0xfff8ffff;
		writel(val,index*0x10+REG_HDMAC_DMAC0_B);
	}

	// Enable all DMA channels
/*	!!MNG, de=1,ds=0,pr=0; 31:29*/
	val=readl(REG_HDMAC_DMACR);
	val &=0x0fffffff;
	val |=0x80000000;
	writel(val,REG_HDMAC_DMACR);
	DPRINTK("DMACR wr:0x%X rb:0x%X\n",val,readl(REG_HDMAC_DMACR));

}
static void hdmac_postenable(struct ata_queued_cmd *qc)
{

	int rw,val;

	switch(qc->tf.command)
	{
		case ATA_CMD_FPDMA_READ://60
		case ATA_CMD_READ:      //c8
		case ATA_CMD_READ_EXT:  //25
			rw=0;
			break;
		case ATA_CMD_FPDMA_WRITE://61
		case ATA_CMD_WRITE:      //ca
		case ATA_CMD_WRITE_EXT:  //35
			rw=1;
			break;
		default:
			break;
	}
	//hdmac_link_cfg(rw);
	writel(0,REG_SATA_LINK_DMACR);
	/*!!MNG, wr=1,rd=0*/
	if(rw>0)
	{
		/*!!MNG, mrd=0,22:16,mwr=126,6:0*/
		val=readl(REG_SATA_LINK_DBTSR);
		val &=0xff80ff80;
		val |=126;
		writel(val,REG_SATA_LINK_DBTSR);
		writel(5,REG_SATA_LINK_DMACR);
	}
	else
	{
		/*!!MNG, mrd=8,mwr=0,*/
		val=readl(REG_SATA_LINK_DBTSR);
		val &=0xff80ff80;
		val |=8<<16;
		writel(val,REG_SATA_LINK_DBTSR);
		writel(2,REG_SATA_LINK_DMACR);
	}
	DPRINTK("hdmac_postenable\n");

}

static void hdmac_cfg(const u32 index,const  struct hdmac_ch_cfg config)
{
	int dmaca,dmacb;
	int virt=IO_PHYS_TO_VIRT(config.destinationAddress);

	writel(config.sourceAddress,REG_HDMAC_DMAC0_SA+0x10*index);
	//writel(virt,REG_HDMAC_DMAC0_DA+0x10*index);
	writel(config.destinationAddress,REG_HDMAC_DMAC0_DA+0x10*index);

	dmaca=readl(REG_HDMAC_DMAC0_A+0x10*index);
	dmaca &=0xff00ffff;
	// Transfer Length HANDLING,beat transfer,reload handling,reload count disable
	dmaca|=config.transferCount-1;
	dmaca|=config.blockCount      <<16;  //16:19
	dmaca|=config.beatType        <<20;  //20:23
	writel(dmaca,REG_HDMAC_DMAC0_A+0x10*index);

	dmacb=readl(REG_HDMAC_DMAC0_B +0x10*index);
	dmacb &=0x3f00ffff;dmacb|=0x00180000;//TT, 31:30=0,16:23=18
	dmacb|=config.transferMode    <<28;  //28:29
	dmacb|=config.transferWidth   <<26;  //26:27
	dmacb|=config.fixedSource     <<25;  //25
	dmacb|=config.fixedDestination<<24;  //24
	DPRINTK("hdmac:wr DMAC0_B=0x%X\n",dmacb);
	writel(dmacb,REG_HDMAC_DMAC0_B+0x10*index);

	dmaca=readl(REG_HDMAC_DMAC0_A+0x10*index);
	dmaca &=0xc0ffffff;
	dmaca|=config.triggerType     <<29;   //29
	dmaca|=config.inputSelect      <<24;  //24:28
	writel(dmaca,REG_HDMAC_DMAC0_A+0x10*index);

	DPRINTK("hdmac:rb DMAC0_B=0x%X\n",readl(REG_HDMAC_DMAC0_B+0x10*index));
	DPRINTK("hdmac_config_channel,(0x%X,%d,0x%X,%d,%d,%d,%d,%d,%d,%d,%d,%d)\n",
			config.sourceAddress,config.fixedSource,virt,config.fixedDestination,
			config.transferMode,config.transferCount,config.blockCount,config.triggerType,config.inputSelect,
			config.transferWidth,config.beatType,config.optTransferType
	);

}
static int hdmac_stop(int index)
{
	int val;
	//disable DMA
	writel(0,REG_HDMAC_DMAC0_A +index*0x10);
	writel(0,REG_HDMAC_DMAC0_B +index*0x10);
	//IRQ disable
	writel(0,REG_HDMAC_DMAC0_B +index*0x10);

	writel(0,REG_HDMAC_DMAC0_SA+index*0x10);
	writel(0,REG_HDMAC_DMAC0_DA+index*0x10);

	val=readl(REG_HDMAC_DMAC0_B+index*0x10);
	val &=0xfff1ffff;
	writel(val,REG_HDMAC_DMAC0_B+index*0x10);
	DPRINTK("hdmac_stop\n");
	return 0;
}

static void hdmac_start(struct ata_queued_cmd *qc)
{
	struct hdmac_ch_cfg config	;
	struct ata_port *ap=qc->ap;
	struct ata_taskfile*tf=&qc->tf;

	unsigned long flags = 0;
	s32       val,index=0;
	s32       command,lba,sectors,feature;
	register u32 end,start;

	config.fixedDestination=0;
	config.triggerType		=HDMAC_TRIGGER_PERIPHERAL;
	config.transferWidth	=HDMAC_WIDTH_LONG_WORD;
	config.transferMode	=HDMAC_BLOCK_MODE;
	config.transferCount	=((qc->nbytes/4)%65536)/8;
	config.blockCount	= qc->nbytes/(4*65536);
	config.beatType	= 13;//HDMAC_BLOCK_MODE;

	DPRINTK("nbytes=0x%d sectsize=%d n_elem=%d tc=%d bc=%d\n",
			qc->nbytes,
			qc->sect_size,
			qc->n_elem,
			config.transferCount,
			config.blockCount
		);

	switch(qc->tf.command)
	{
		case ATA_CMD_FPDMA_READ://60
		case ATA_CMD_READ:		//c8
		case ATA_CMD_READ_EXT:	//25
			config.optTransferType   =2;
			config.fixedSource       =1;//rd =1, wr =0
			config.sourceAddress     =REG_SATA_LINK_DMADR_R;
			config.inputSelect       =HDMAC_TRIGGER_INPUT_SATA_RX;
			config.destinationAddress=qc->sg->dma_address;
			break;
		case ATA_CMD_FPDMA_WRITE://61
		case ATA_CMD_WRITE:      //ca
		case ATA_CMD_WRITE_EXT:  //35
			config.optTransferType=3;
			config.fixedSource    =0; //rd =1, wr =0
			config.sourceAddress  =qc->sg->dma_address;
			config.inputSelect    =HDMAC_TRIGGER_INPUT_SATA_TX;
			config.destinationAddress=REG_SATA_LINK_DMADR_R;
			break;
		default:
			break;
	}

	DPRINTK("hdmac_start, cmd=0x%X\n",qc->tf.command);

	val=readl( REG_HDMAC_DMAC0_A+index*0x10);
	val&=0x7fffffff;
	writel(val,REG_HDMAC_DMAC0_A+index*0x10);
	start=config.destinationAddress;

	end = ( start + 1024) - 1;
	start=start & ~0x3;
	end = end  & ~0x3;
	spin_lock_irqsave(&qc->ap->host->lock, flags);
	//asm volatile( "mcrr p15,0,%0,%1,c12" : : "r"(end), "r"(start) );
	hdmac_cfg(index,config);
	spin_unlock_irqrestore(&qc->ap->host->lock, flags);
	val=readl(REG_HDMAC_DMAC0_A+index*0x10);
	val|=0x80000000;
	writel(val,REG_HDMAC_DMAC0_A+index*0x10);
	DPRINTK("DMAC0_A wr:0x%X,rb:0x%X\n",val,readl(REG_HDMAC_DMAC0_A+index*0x10));
	hdmac_postenable(qc);
	//break;

	ap->ops->sff_tf_load(ap, tf);
	ap->ops->sff_exec_command(ap, tf);
	command = tf->command;
	lba    	 = 0;
	sectors = 1;
	feature = 0;
	//ret=send_cmd(port,timeout,command,lba,sectors,feature);
	DPRINTK("hdmac_start <<>> exec command \n");

}
/**
 *      fst_platform_probe - handle a positive probe of an soc Marvell
 *      host
 *      @pdev: platform device found
 *
 *      LOCKING:
 *      Inherited from caller.
 */
void hdmac_irq()
{
	DPRINTK(">> hdmac_irq << \n");
}
#include <linux/sysdev.h>
static struct sysdev_class dma_sysclass = {
	.name= "mb86hxx-hdma",
};
struct mb86hxx_dmac{
	struct sys_device	 sysdev;
};
static int fst_platform_probe(struct platform_device *pdev)
{
	static int 	printed_version;
	s32         n_ports,rc,irq;
	const  struct ata_port_info *ppi[] = { &fst_port_info[0], NULL };
	struct ata_host      *host;
	struct mb86hxx_dmac 	*dmac;
	struct host_priv     *hp;

	if (!printed_version++)
		dev_printk(KERN_INFO, &pdev->dev, "version " FST_DRV_VER "\n");
	n_ports =1;
	host 	= ata_host_alloc_pinfo(&pdev->dev, ppi, n_ports);
	hp		=devm_kzalloc(&pdev->dev,sizeof(*hp),GFP_KERNEL);
	hp->sg_tbl_pool=dmam_pool_create("sg_tbl", &pdev->dev, 4096,
									4096, 0);
	DPRINTK("hp->sg_tbl_pool:0x%X\n",hp->sg_tbl_pool);
	host->private_data=hp;

	rc = fst_init_host(host);
	if (rc)
		return rc;
	rc = phy_init();
	hdmac_init();
	dmac = kzalloc(sizeof(struct mb86hxx_dmac), GFP_KERNEL);
	DPRINTK("phy_init ret=0x%X\n",rc);
	dmac->sysdev.id  = 0 / 8;
	dmac->sysdev.cls = &dma_sysclass;
	rc = sysdev_class_register(&dma_sysclass);
	rc = sysdev_register(&dmac->sysdev);
	rc = request_irq(16, hdmac_irq, IRQF_SHARED, "HDMA",dmac );
	DPRINTK("request irq ret=%d\n",rc);
	dev_printk(KERN_INFO, &pdev->dev,
		    "ports %d\n",
		    n_ports);
	irq = 9;
	rc=ata_host_activate(host, irq, fst_interrupt,
				 IRQF_SHARED, &fst_sht);
	return  rc;
}

/*
 *
 *      fst_platform_remove    -       unplug a platform interface
 *      @pdev: platform device
 *
 *      A platform bus SATA device has been unplugged. Perform the needed
 *      cleanup. Also called on module unload for any active devices.
 */
static int __devexit fst_platform_remove(struct platform_device *pdev)
{
	struct device *dev    = &pdev->dev;
	struct ata_host *host = dev_get_drvdata(dev);
	ata_host_detach(host);
	return 0;
}

static struct platform_driver fst_platform_driver = {
	.probe		= fst_platform_probe,
	.remove		= __devexit_p(fst_platform_remove),
	.driver		= {
				   .name = FST_DRV_NAME,
				   .owner = THIS_MODULE,
				    },
};


static struct platform_device *fst_scsi_device;

static int    fst_platform_probe(struct platform_device *pdev);
static int __devexit fst_platform_remove(struct platform_device *pdev);

static int __init fst_init(void)
{
	int rc = -ENODEV;

	rc = platform_driver_register(&fst_platform_driver);
	fst_scsi_device = platform_device_register_simple("fst-sata",-1, NULL, 0);
	if (IS_ERR(fst_scsi_device)) {
		platform_device_unregister(fst_scsi_device);
		return PTR_ERR(fst_scsi_device);
	}

	return rc;
}

static void __exit fst_exit(void)
{
	platform_driver_unregister(&fst_platform_driver);
}

MODULE_AUTHOR("M. N. G");
MODULE_DESCRIPTION("SCSI low-level driver for SATA controllers");
MODULE_LICENSE("GPL");
MODULE_VERSION(FST_DRV_VER);
MODULE_ALIAS("platform:mb86h61" FST_DRV_NAME);

module_init(fst_init);
module_exit(fst_exit);

