/*
 *  drivers/mtd/devices/mb86hxx.c
 *
 * Copyright (C) 2008 FUJITSU MICROELECTRONICS LIMITED
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/clk.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/dma-mapping.h>
#include <linux/err.h>
#include <linux/interrupt.h>
#include <linux/spi/spi.h>

#include <asm/io.h>
#include <linux/spi/spi.h>

/*
 * The core SPI transfer engine just talks to a register bank to set up
 * DMA transfers; transfer queue progress is driven by IRQs.  The clock
 * framework provides the base clock, subdivided for each spi_device.
 */
struct mb86hxx_spi {
	spinlock_t		lock;

	void __iomem		*regs;
	int			irq;
	struct clk		*clk;
	struct platform_device	*pdev;
	struct spi_device	*stay;

	u8			stopping;
	struct list_head	queue;
	struct spi_transfer	*current_transfer;
	unsigned long		current_remaining_bytes;
	struct spi_transfer	*next_transfer;
	unsigned long		next_remaining_bytes;

	void			*buffer;
	dma_addr_t		buffer_dma;
};

static struct platform_driver mb86hxx_spi_driver = {
	.driver		= {
		.name	= "mb86hxx_spi",
		.owner	= THIS_MODULE,
	},
};
static int mb86hxx_spi_setup(struct spi_device *spi);
static int __init mb86hxx_spi_probe(struct platform_device *pdev)
{
//	struct resource		*regs;
//	int			irq;
//	struct clk		*clk;
	int			ret;
	struct spi_master	*master;
	struct mb86hxx_spi	*as;

//	regs = platform_get_resource(pdev, IORESOURCE_MEM, 0);
//	if (!regs)
//		return -ENXIO;
//
//	irq = platform_get_irq(pdev, 0);
//	if (irq < 0)
//		return irq;
//
//	clk = clk_get(&pdev->dev, "spi_clk");
//	if (IS_ERR(clk))
//		return PTR_ERR(clk);
	/* setup spi core then -specific driver state */
	ret = -ENOMEM;
	master = spi_alloc_master(&pdev->dev, sizeof (*as));
	if (!master)
		goto out_free;
	
	/* the spi->mode bits understood by this driver: */
	master->mode_bits = SPI_CPOL | SPI_CPHA | SPI_CS_HIGH;

	master->bus_num = pdev->id;
	master->num_chipselect = 4;
	master->setup = mb86hxx_spi_setup;		
	master->transfer = 0;	
	master->cleanup = 0;	
	platform_set_drvdata(pdev, master);

	as = spi_master_get_devdata(master);
	/*
	 * Scratch buffer is used for throwaway rx and tx data.
	 * It's coherent to minimize dcache pollution.
	 */
//	as->buffer = dma_alloc_coherent(&pdev->dev, BUFFER_SIZE,
//					&as->buffer_dma, GFP_KERNEL);
//	if (!as->buffer)
//		goto out_free;

//	spin_lock_init(&as->lock);
//	INIT_LIST_HEAD(&as->queue);
//	as->pdev = pdev;
//	as->regs = ioremap(regs->start, (regs->end - regs->start) + 1);
//	if (!as->regs)
//		goto out_free_buffer;
//	as->irq = irq;
//	as->clk = clk;
//
//	ret = request_irq(irq, _spi_interrupt, 0,
//			dev_name(&pdev->dev), master);
//	if (ret)
//		goto out_unmap_regs;
//
//	/* Initialize the hardware */
//	clk_enable(clk);
//	spi_writel(as, CR, SPI_BIT(SWRST));
//	spi_writel(as, CR, SPI_BIT(SWRST)); /* AT91SAM9263 Rev B workaround */
//	spi_writel(as, MR, SPI_BIT(MSTR) | SPI_BIT(MODFDIS));
//	spi_writel(as, PTCR, SPI_BIT(RXTDIS) | SPI_BIT(TXTDIS));
//	spi_writel(as, CR, SPI_BIT(SPIEN));

	/* go! */
//	dev_info(&pdev->dev, "Fujitsu SPI Controller at 0x%08lx (irq %d)\n",
//			(unsigned long)regs->start, 0);

	ret = spi_register_master(master);
	if (ret)
		goto out_reset_hw;
	return 0;

out_reset_hw:
//	spi_writel(as, CR, SPI_BIT(SWRST));
//	spi_writel(as, CR, SPI_BIT(SWRST)); /* AT91SAM9263 Rev B workaround */
//	clk_disable(clk);
//	free_irq(irq, master);
//out_unmap_regs:
//	iounmap(as->regs);
//out_free_buffer:
//	dma_free_coherent(&pdev->dev, BUFFER_SIZE, as->buffer,
//			as->buffer_dma);
out_free:
//	clk_put(clk);
	spi_master_put(master);
	return ret;
}
static int mb86hxx_spi_setup(struct spi_device *spi)
{
	struct mb86hxx_spi	*as;
//	struct _spi_device	*asd;
//	u32			scbr, csr;
//	unsigned int		bits = spi->bits_per_word;
//	unsigned long		bus_hz;
//	unsigned int		npcs_pin;
//	int			ret;

	as = spi_master_get_devdata(spi->master);

	if (as->stopping)
		return -ESHUTDOWN;

	return 0;
}

static int __init mb86hxx_spi_init(void)
{
    return platform_driver_probe(&mb86hxx_spi_driver, mb86hxx_spi_probe);
}
static void __exit mb86hxx_spi_exit(void)
{
    platform_driver_unregister(&mb86hxx_spi_driver);    
}

module_init(mb86hxx_spi_init);
module_exit(mb86hxx_spi_exit);

 
