/*
 * Copyright (C) 2009 by FUJITSU MICROELECTRONICS LIMITED
 *
 * The Inventra Controller Driver for Linux is free software; you
 * can redistribute it and/or modify it under the terms of the GNU
 * General Public License version 2 as published by the Free Software
 * Foundation.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/clk.h>
#include <mach/hardware.h>
#include <asm/io.h>

#include "musb_core.h"


static int dma_off = 0;

int musb_platform_set_mode(struct musb *musb, u8 musb_mode)
{
	return 0;
}

int __init musb_platform_init(struct musb *musb)
{
	/* aka, I don't know why.
	 * But otherwise, it fails in musb_init_controller(). */
        usb_nop_xceiv_register();
        musb->xceiv = otg_get_transceiver();
        if (!musb->xceiv)
                return -ENODEV;

        /* start clock */
        musb->clock = clk_get((struct device *)musb->controller, "usbhs_ick");
        clk_enable(musb->clock);

        return 0;
}

int musb_platform_exit(struct musb *musb)
{
	clk_disable(musb->clock);

	return 0;
}

void musb_platform_enable(struct musb *musb)
{
	if (is_dma_capable() && dma_off)
		printk(KERN_WARNING "%s %s: dma not reactivated\n",
				__FILE__, __FUNCTION__);
	else
		dma_off = 1;
}

void musb_platform_disable(struct musb *musb)
{
	if (is_dma_capable()) {
		printk(KERN_WARNING "%s %s: dma still active\n",
				__FILE__, __FUNCTION__);
		dma_off = 1;
	}
}
