/*
 * MUSB OTG driver register I/O
 *
 * Copyright 2005 Mentor Graphics Corporation
 * Copyright (C) 2005-2006 by Texas Instruments
 * Copyright (C) 2006-2007 Nokia Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __MUSB_LINUX_PLATFORM_ARCH_H__
#define __MUSB_LINUX_PLATFORM_ARCH_H__

#include <linux/io.h>
#ifdef CONFIG_ARCH_MB86HXX
#include "mb86hxx.h"
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
extern volatile u8 usb_mode;
extern volatile u8 usb_mode1;
#else
extern volatile u32 usb_mode;
extern volatile u32 usb_mode1;
#endif
extern volatile void __iomem *usb_base;
extern volatile void __iomem *usb_base1;

#endif /* CONFIG_ARCH_MB86HXX */

#if !defined(CONFIG_ARM) && !defined(CONFIG_SUPERH) \
	&& !defined(CONFIG_AVR32) && !defined(CONFIG_PPC32) \
	&& !defined(CONFIG_PPC64) && !defined(CONFIG_BLACKFIN)
static inline void readsl(const void __iomem *addr, void *buf, int len)
	{ insl((unsigned long)addr, buf, len); }
static inline void readsw(const void __iomem *addr, void *buf, int len)
	{ insw((unsigned long)addr, buf, len); }
static inline void readsb(const void __iomem *addr, void *buf, int len)
	{ insb((unsigned long)addr, buf, len); }

static inline void writesl(const void __iomem *addr, const void *buf, int len)
	{ outsl((unsigned long)addr, buf, len); }
static inline void writesw(const void __iomem *addr, const void *buf, int len)
	{ outsw((unsigned long)addr, buf, len); }
static inline void writesb(const void __iomem *addr, const void *buf, int len)
	{ outsb((unsigned long)addr, buf, len); }

#endif

#ifndef CONFIG_BLACKFIN

/* NOTE:  these offsets are all in bytes */

#ifdef CONFIG_ARCH_MB86HXX
static inline u16 musb_readw(const void __iomem *addr, unsigned offset)
{
        u16 data;

#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
        mb86hxx_writeb(MB86HXX_USB_2BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB_MODE);
#else
        if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
            usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_2BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB0_MODE);
        }else if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
            usb_mode1 &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_2BYTE_ACCESS<<1 | usb_mode1, MB86HXX_USB1_MODE);
        }
#endif
    data = __raw_readw(addr + (offset<<2));
        return ( data );
}

static inline u32 musb_readl(const void __iomem *addr, unsigned offset)
{
        u32 data;

#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
        mb86hxx_writeb(MB86HXX_USB_4BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB_MODE);
#else
        if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
            usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_4BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB0_MODE);
        }else if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
            usb_mode1 &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_4BYTE_ACCESS<<1 | usb_mode1, MB86HXX_USB1_MODE);
        }
#endif
        data = __raw_readl(addr + (offset<<2));
        return ( data );
}


static inline void musb_writew(void __iomem *addr, unsigned offset, u16 data)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
        mb86hxx_writeb(MB86HXX_USB_2BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB_MODE);
#else
        if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
            usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_2BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB0_MODE);
        }else if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
            usb_mode1 &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_2BYTE_ACCESS<<1 | usb_mode1, MB86HXX_USB1_MODE);
        }
#endif
        __raw_writew(data, addr + (offset<<2));
}

static inline void musb_writel(void __iomem *addr, unsigned offset, u32 data)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
        mb86hxx_writeb(MB86HXX_USB_4BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB_MODE);
#else
        if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
            usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_4BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB0_MODE);
        }else if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
            usb_mode1 &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_4BYTE_ACCESS<<1 | usb_mode1, MB86HXX_USB1_MODE);
        }
#endif
        __raw_writel(data, addr + (offset<<2));
}
#else
static inline u16 musb_readw(const void __iomem *addr, unsigned offset)
	{ return __raw_readw(addr + offset); }

static inline u32 musb_readl(const void __iomem *addr, unsigned offset)
	{ return __raw_readl(addr + offset); }


static inline void musb_writew(void __iomem *addr, unsigned offset, u16 data)
	{ __raw_writew(data, addr + offset); }

static inline void musb_writel(void __iomem *addr, unsigned offset, u32 data)
	{ __raw_writel(data, addr + offset); }

#endif /* CONFIG_ARCH_MB86HXX */

#ifdef CONFIG_USB_TUSB6010

/*
 * TUSB6010 doesn't allow 8-bit access; 16-bit access is the minimum.
 */
static inline u8 musb_readb(const void __iomem *addr, unsigned offset)
{
	u16 tmp;
	u8 val;

	tmp = __raw_readw(addr + (offset & ~1));
	if (offset & 1)
		val = (tmp >> 8);
	else
		val = tmp & 0xff;

	return val;
}

static inline void musb_writeb(void __iomem *addr, unsigned offset, u8 data)
{
	u16 tmp;

	tmp = __raw_readw(addr + (offset & ~1));
	if (offset & 1)
		tmp = (data << 8) | (tmp & 0xff);
	else
		tmp = (tmp & 0xff00) | data;

	__raw_writew(tmp, addr + (offset & ~1));
}

#else
#ifdef CONFIG_ARCH_MB86HXX
static inline u8 musb_readb(const void __iomem *addr, unsigned offset)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
        mb86hxx_writeb(MB86HXX_USB_1BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB_MODE);
#else
        if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
            usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_1BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB0_MODE);
        }else if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
            usb_mode1 &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_1BYTE_ACCESS<<1 | usb_mode1, MB86HXX_USB1_MODE);
        }
#endif
        return __raw_readb(addr + (offset<<2));
}

static inline void musb_writeb(void __iomem *addr, unsigned offset, u8 data)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
        mb86hxx_writeb(MB86HXX_USB_1BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB_MODE);
#else
        if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
            usb_mode &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_1BYTE_ACCESS<<1 | usb_mode, MB86HXX_USB0_MODE);
        }else if(((unsigned long)addr & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
            usb_mode1 &= ~MB86HXX_USB_LENGTH_MASK;
            mb86hxx_writel(MB86HXX_USB_1BYTE_ACCESS<<1 | usb_mode1, MB86HXX_USB1_MODE);
        }
#endif
        __raw_writeb(data, addr + (offset<<2));
}
#else

static inline u8 musb_readb(const void __iomem *addr, unsigned offset)
	{ return __raw_readb(addr + offset); }

static inline void musb_writeb(void __iomem *addr, unsigned offset, u8 data)
	{ __raw_writeb(data, addr + offset); }

#endif  /* CONFIG_ARCH_MB86HXX */
#endif	/* CONFIG_USB_TUSB6010 */

#else

static inline u8 musb_readb(const void __iomem *addr, unsigned offset)
	{ return (u8) (bfin_read16(addr + offset)); }

static inline u16 musb_readw(const void __iomem *addr, unsigned offset)
	{ return bfin_read16(addr + offset); }

static inline u32 musb_readl(const void __iomem *addr, unsigned offset)
	{ return (u32) (bfin_read16(addr + offset)); }

static inline void musb_writeb(void __iomem *addr, unsigned offset, u8 data)
	{ bfin_write16(addr + offset, (u16) data); }

static inline void musb_writew(void __iomem *addr, unsigned offset, u16 data)
	{ bfin_write16(addr + offset, data); }

static inline void musb_writel(void __iomem *addr, unsigned offset, u32 data)
	{ bfin_write16(addr + offset, (u16) data); }

#endif /* CONFIG_BLACKFIN */

#endif
