/*
 * MUSB OTG driver - support for Mentor's DMA controller
 *
 * Copyright 2005 Mentor Graphics Corporation
 * Copyright (C) 2005-2007 by Texas Instruments
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include <linux/device.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include "musb_core.h"
#include "musbhsdma.h"
#ifdef CONFIG_ARCH_MB86HXX
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
extern volatile u8 usb_mode;
extern volatile u8 usb_mode1;
u8     irq_disabled = 0;
#else
extern volatile u32 usb_mode;
extern volatile u32 usb_mode1;
#endif

extern volatile void __iomem *usb_base;
extern volatile void __iomem *usb_base1;

u8     initialize = 0;
struct semaphore dma_sem;
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
static u8     initialize1 = 0;
struct semaphore dma_sem1;
#endif
#endif /* CONFIG_ARCH_MB86HXX */

static int dma_controller_start(struct dma_controller *c)
{
	/* nothing to do */
	return 0;
}

static void dma_channel_release(struct dma_channel *channel);

#ifdef CONFIG_ARCH_MB86HXX
static void dma_prepare(FAPI_DMA_RequestT* pDmaRequest);
static void dma_callback(FAPI_DMA_RequestT* pDmaRequest);
static void dma_request_send(struct musb_dma_channel *pImplChannel, FAPI_DMA_RequestT* pDmaRequest, dma_addr_t dma_addr, u32 len);
#endif /* CONFIG_ARCH_MB86HXX */

static int dma_controller_stop(struct dma_controller *c)
{
	struct musb_dma_controller *controller = container_of(c,
			struct musb_dma_controller, controller);
	struct musb *musb = controller->private_data;
	struct dma_channel *channel;
	u8 bit;

	if (controller->used_channels != 0) {
		dev_err(musb->controller,
			"Stopping DMA controller while channel active\n");

		for (bit = 0; bit < MUSB_HSDMA_CHANNELS; bit++) {
			if (controller->used_channels & (1 << bit)) {
				channel = &controller->channel[bit].channel;
				dma_channel_release(channel);

				if (!controller->used_channels)
					break;
			}
		}
	}

	return 0;
}

static struct dma_channel *dma_channel_allocate(struct dma_controller *c,
				struct musb_hw_ep *hw_ep, u8 transmit)
{
	struct musb_dma_controller *controller = container_of(c,
			struct musb_dma_controller, controller);
	struct musb_dma_channel *musb_channel = NULL;
	struct dma_channel *channel = NULL;
	u8 bit;
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
	unsigned long mbase;
#endif

	for (bit = 0; bit < MUSB_HSDMA_CHANNELS; bit++) {
		if (!(controller->used_channels & (1 << bit))) {
			controller->used_channels |= (1 << bit);
			musb_channel = &(controller->channel[bit]);
			musb_channel->controller = controller;
			musb_channel->idx = bit;
			musb_channel->epnum = hw_ep->epnum;
			musb_channel->transmit = transmit;
			channel = &(musb_channel->channel);
			channel->private_data = musb_channel;
			channel->status = MUSB_DMA_STATUS_FREE;
			channel->max_len = 0x10000;
			/* Tx => mode 1; Rx => mode 0 */
			channel->desired_mode = transmit;
			channel->actual_len = 0;
			break;
		}
	}

#ifdef CONFIG_ARCH_MB86HXX
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
	if (musb_channel) {
		if(!initialize){
			sema_init(&dma_sem, 1);
			initialize = 1;
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
			if(irq_disabled){
				mb86hxx_writeb(0x7e, MB86HXX_USB_INT_MASK);
				irq_disabled = 0;
			}
#endif /* CONFIG_MACH_FUJITSU_TVSTBSOC */
		}
		musb_channel->dma_request = dma_funcStr.AllocateRequest(
											FAPI_DMA_FEATURE_USB_COPY, 0);
	}
#else
	mbase = (unsigned long)musb_channel->controller->base;

#ifdef CONFIG_MB86HXX_USB_CORE0
	if (musb_channel && ((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base )) {
		if(!initialize){
			sema_init(&dma_sem, 1);
			initialize = 1;
		}
		musb_channel->dma_request = dma_funcStr.AllocateRequest(
											FAPI_DMA_FEATURE_USB_COPY, 0);
	}
#endif /* CONFIG_MB86HXX_USB_CORE0 */
#ifdef CONFIG_MB86HXX_USB_CORE1
	if (musb_channel && ((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 )) {
		if(!initialize1){
			sema_init(&dma_sem1, 1);
			initialize1 = 1;
		}
		musb_channel->dma_request = dma_funcStr.AllocateRequest(
											FAPI_DMA_FEATURE_USB2_COPY, 0);
	}
#endif /* CONFIG_MB86HXX_USB_CORE1 */
#endif /* CONFIG_MACH_FUJITSU_TVSTBSOC */
#endif /* CONFIG_ARCH_MB86HXX */

	return channel;
}

static void dma_channel_release(struct dma_channel *channel)
{
	struct musb_dma_channel *musb_channel = channel->private_data;
	FAPI_DMA_RequestT* dma_request_addr;

	dma_request_addr = musb_channel->dma_request;

	channel->actual_len = 0;
	musb_channel->start_addr = 0;
	musb_channel->len = 0;

	musb_channel->controller->used_channels &=
		~(1 << musb_channel->idx);

	channel->status = MUSB_DMA_STATUS_UNKNOWN;
#ifdef CONFIG_ARCH_MB86HXX
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
	if(irq_disabled){
		mb86hxx_writeb(0x7e, MB86HXX_USB_INT_MASK);
		irq_disabled = 0;
	}
#endif /* CONFIG_MACH_FUJITSU_TVSTBSOC */
	if (musb_channel) {
		dma_funcStr.CancelRequest(dma_request_addr,
									dma_request_addr->feature);
		dma_funcStr.ReleaseRequest(dma_request_addr);
	}
#endif /* CONFIG_ARCH_MB86HXX */
}

#ifndef CONFIG_ARCH_MB86HXX
static void configure_channel(struct dma_channel *channel,
				u16 packet_sz, u8 mode,
				dma_addr_t dma_addr, u32 len)
{
	struct musb_dma_channel *musb_channel = channel->private_data;
	struct musb_dma_controller *controller = musb_channel->controller;
	void __iomem *mbase = controller->base;
	u8 bchannel = musb_channel->idx;
	u16 csr = 0;

	DBG(4, "%p, pkt_sz %d, addr 0x%x, len %d, mode %d\n",
			channel, packet_sz, dma_addr, len, mode);

	if (mode) {
		csr |= 1 << MUSB_HSDMA_MODE1_SHIFT;
		BUG_ON(len < packet_sz);

		if (packet_sz >= 64) {
			csr |= MUSB_HSDMA_BURSTMODE_INCR16
					<< MUSB_HSDMA_BURSTMODE_SHIFT;
		} else if (packet_sz >= 32) {
			csr |= MUSB_HSDMA_BURSTMODE_INCR8
					<< MUSB_HSDMA_BURSTMODE_SHIFT;
		} else if (packet_sz >= 16) {
			csr |= MUSB_HSDMA_BURSTMODE_INCR4
					<< MUSB_HSDMA_BURSTMODE_SHIFT;
		}
	}

	csr |= (musb_channel->epnum << MUSB_HSDMA_ENDPOINT_SHIFT)
		| (1 << MUSB_HSDMA_ENABLE_SHIFT)
		| (1 << MUSB_HSDMA_IRQENABLE_SHIFT)
		| (musb_channel->transmit
				? (1 << MUSB_HSDMA_TRANSMIT_SHIFT)
				: 0);

	/* address/count */
	musb_write_hsdma_addr(mbase, bchannel, dma_addr);
	musb_write_hsdma_count(mbase, bchannel, len);

	/* control (this should start things) */
	musb_writew(mbase,
		MUSB_HSDMA_CHANNEL_OFFSET(bchannel, MUSB_HSDMA_CONTROL),
		csr);
}
#endif /* CONFIG_ARCH_MB86HXX */

static int dma_channel_program(struct dma_channel *channel,
				u16 packet_sz, u8 mode,
				dma_addr_t dma_addr, u32 len)
{
	struct musb_dma_channel *musb_channel = channel->private_data;
#ifdef CONFIG_ARCH_MB86HXX
	FAPI_DMA_RequestT* dma_request_addr;

	dma_request_addr = musb_channel->dma_request;

#endif /* CONFIG_ARCH_MB86HXX */

	DBG(2, "ep%d-%s pkt_sz %d, dma_addr 0x%x length %d, mode %d\n",
		musb_channel->epnum,
		musb_channel->transmit ? "Tx" : "Rx",
		packet_sz, dma_addr, len, mode);

	BUG_ON(channel->status == MUSB_DMA_STATUS_UNKNOWN ||
		channel->status == MUSB_DMA_STATUS_BUSY);

#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
	if(musb_channel->transmit == 0){
		/* disable general USB interrupt */
		mb86hxx_writeb(0x7f, MB86HXX_USB_INT_MASK);
		irq_disabled += 0x10;
	}
#endif

	channel->actual_len = 0;
	musb_channel->start_addr = dma_addr;
	musb_channel->len = len;
	musb_channel->max_packet_sz = packet_sz;
	channel->status = MUSB_DMA_STATUS_BUSY;

#ifndef CONFIG_ARCH_MB86HXX
	if ((mode == 1) && (len >= packet_sz))
		configure_channel(channel, packet_sz, 1, dma_addr, len);
	else
		configure_channel(channel, packet_sz, 0, dma_addr, len);
#else
	if ((mode == 1) && (musb_channel->transmit == 0) && (len % packet_sz == 0)){
			len -= packet_sz;
	}
	dma_request_send(musb_channel, dma_request_addr, dma_addr, len);
#endif /* CONFIG_ARCH_MB86HXX */

	return true;
}

static int dma_channel_abort(struct dma_channel *channel)
{
	struct musb_dma_channel *musb_channel = channel->private_data;
	void __iomem *mbase = musb_channel->controller->base;

	u8 bchannel = musb_channel->idx;
	int offset;
	u16 csr;

	if (channel->status == MUSB_DMA_STATUS_BUSY) {
		if (musb_channel->transmit) {
			offset = MUSB_EP_OFFSET(musb_channel->epnum,
						MUSB_TXCSR);

			/*
			 * The programming guide says that we must clear
			 * the DMAENAB bit before the DMAMODE bit...
			 */
			csr = musb_readw(mbase, offset);
			csr &= ~(MUSB_TXCSR_AUTOSET | MUSB_TXCSR_DMAENAB);
			musb_writew(mbase, offset, csr);
			csr &= ~MUSB_TXCSR_DMAMODE;
			musb_writew(mbase, offset, csr);
		} else {
			offset = MUSB_EP_OFFSET(musb_channel->epnum,
						MUSB_RXCSR);

			csr = musb_readw(mbase, offset);
			csr &= ~(MUSB_RXCSR_AUTOCLEAR |
				 MUSB_RXCSR_DMAENAB |
				 MUSB_RXCSR_DMAMODE);
			musb_writew(mbase, offset, csr);
		}

		musb_writew(mbase,
			MUSB_HSDMA_CHANNEL_OFFSET(bchannel, MUSB_HSDMA_CONTROL),
			0);
		musb_write_hsdma_addr(mbase, bchannel, 0);
		musb_write_hsdma_count(mbase, bchannel, 0);
		channel->status = MUSB_DMA_STATUS_FREE;
	}

	return 0;
}

#ifndef CONFIG_ARCH_MB86HXX
static irqreturn_t dma_controller_irq(int irq, void *private_data)
{
	struct musb_dma_controller *controller = private_data;
	struct musb *musb = controller->private_data;
	struct musb_dma_channel *musb_channel;
	struct dma_channel *channel;

	void __iomem *mbase = controller->base;

	irqreturn_t retval = IRQ_NONE;

	unsigned long flags;

	u8 bchannel;
	u8 int_hsdma;

	u32 addr;
	u16 csr;

	spin_lock_irqsave(&musb->lock, flags);

	int_hsdma = musb_readb(mbase, MUSB_HSDMA_INTR);
	if (!int_hsdma)
		goto done;

	for (bchannel = 0; bchannel < MUSB_HSDMA_CHANNELS; bchannel++) {
		if (int_hsdma & (1 << bchannel)) {
			musb_channel = (struct musb_dma_channel *)
					&(controller->channel[bchannel]);
			channel = &musb_channel->channel;

			csr = musb_readw(mbase,
					MUSB_HSDMA_CHANNEL_OFFSET(bchannel,
							MUSB_HSDMA_CONTROL));

			if (csr & (1 << MUSB_HSDMA_BUSERROR_SHIFT)) {
				musb_channel->channel.status =
					MUSB_DMA_STATUS_BUS_ABORT;
			} else {
				u8 devctl;

				addr = musb_read_hsdma_addr(mbase,
						bchannel);
				channel->actual_len = addr
					- musb_channel->start_addr;

				DBG(2, "ch %p, 0x%x -> 0x%x (%d / %d) %s\n",
					channel, musb_channel->start_addr,
					addr, channel->actual_len,
					musb_channel->len,
					(channel->actual_len
						< musb_channel->len) ?
					"=> reconfig 0" : "=> complete");

				devctl = musb_readb(mbase, MUSB_DEVCTL);

				channel->status = MUSB_DMA_STATUS_FREE;

				/* completed */
				if ((devctl & MUSB_DEVCTL_HM)
					&& (musb_channel->transmit)
					&& ((channel->desired_mode == 0)
					    || (channel->actual_len &
					    (musb_channel->max_packet_sz - 1)))
				    ) {
					u8  epnum  = musb_channel->epnum;
					int offset = MUSB_EP_OFFSET(epnum,
								    MUSB_TXCSR);
					u16 txcsr;

					/*
					 * The programming guide says that we
					 * must clear DMAENAB before DMAMODE.
					 */
					musb_ep_select(mbase, epnum);
					txcsr = musb_readw(mbase, offset);
					txcsr &= ~(MUSB_TXCSR_DMAENAB
							| MUSB_TXCSR_AUTOSET);
					musb_writew(mbase, offset, txcsr);
					/* Send out the packet */
					txcsr &= ~MUSB_TXCSR_DMAMODE;
					txcsr |=  MUSB_TXCSR_TXPKTRDY;
					musb_writew(mbase, offset, txcsr);
				}
				musb_dma_completion(musb, musb_channel->epnum,
						    musb_channel->transmit);
			}
		}
	}

#ifdef CONFIG_BLACKFIN
	/* Clear DMA interrup flags */
	musb_writeb(mbase, MUSB_HSDMA_INTR, int_hsdma);
#endif

	retval = IRQ_HANDLED;
done:
	spin_unlock_irqrestore(&musb->lock, flags);
	return retval;
}
#endif /* CONFIG_ARCH_MB86HXX */

void dma_controller_destroy(struct dma_controller *c)
{
	struct musb_dma_controller *controller = container_of(c,
			struct musb_dma_controller, controller);

	if (!controller)
		return;

	if (controller->irq)
		free_irq(controller->irq, c);

	kfree(controller);
}

struct dma_controller *__init
dma_controller_create(struct musb *musb, void __iomem *base)
{
	struct musb_dma_controller *controller;
	struct device *dev = musb->controller;
	struct platform_device *pdev = to_platform_device(dev);
	int irq = platform_get_irq(pdev, 1);

	if (irq == 0) {
		dev_err(dev, "No DMA interrupt line!\n");
		return NULL;
	}

	controller = kzalloc(sizeof(*controller), GFP_KERNEL);
	if (!controller)
		return NULL;

	controller->channel_count = MUSB_HSDMA_CHANNELS;
	controller->private_data = musb;
	controller->base = base;

	controller->controller.start = dma_controller_start;
	controller->controller.stop = dma_controller_stop;
	controller->controller.channel_alloc = dma_channel_allocate;
	controller->controller.channel_release = dma_channel_release;
	controller->controller.channel_program = dma_channel_program;
	controller->controller.channel_abort = dma_channel_abort;

#ifndef CONFIG_ARCH_MB86HXX
	if (request_irq(irq, dma_controller_irq, IRQF_DISABLED,
			dev_name(musb->controller), &controller->controller)) {
		dev_err(dev, "request_irq %d failed!\n", irq);
		dma_controller_destroy(&controller->controller);

		return NULL;
	}
#endif /* CONFIG_ARCH_MB86HXX */

	controller->irq = irq;

	return &controller->controller;
}

#ifdef CONFIG_ARCH_MB86HXX
static void dma_prepare(FAPI_DMA_RequestT* pDmaRequest)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
    struct musb_dma_channel *pDmaChannel = (struct musb_dma_channel*)pDmaRequest->optData;
#else
    struct musb_dma_channel *pDmaChannel = (struct musb_dma_channel*)pDmaRequest->postEnableOptDataPtr;
#endif
    u8 desired_mode = pDmaChannel->channel.desired_mode;
	struct musb *musb = pDmaChannel->controller->private_data;
	unsigned long flags = 0;
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
	unsigned long mbase = (unsigned long)pDmaChannel->controller->base;
#endif

	spin_lock_irqsave(&musb->lock, flags);

#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
	if((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){
		down_interruptible(&dma_sem);
	}else if((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
		down_interruptible(&dma_sem1);
	}
#endif

#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
    if ( pDmaChannel->transmit ) {
		/* Paced EP Write Channel */
		usb_mode &= 0xe7;
		if(desired_mode)
			usb_mode |= pDmaChannel->epnum<<3;
    } else {
		/* Paced EP Read Channel */
		usb_mode &= 0x9f;
		if(desired_mode)
			usb_mode |= pDmaChannel->epnum<<5;
    }
	mb86hxx_writeb(usb_mode, MB86HXX_USB_MODE);
	DBG(7, "usb_mode %x\n", usb_mode);
#else
    if((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base ){

	    if ( pDmaChannel->transmit ) {
			/* Paced EP Write Channel */
			usb_mode &= 0xe7;
			if(desired_mode)
				usb_mode |= pDmaChannel->epnum<<3;
    	} else {
			/* Paced EP Read Channel */
			usb_mode &= 0x9f;
			if(desired_mode)
				usb_mode |= pDmaChannel->epnum<<5;
    	}
		mb86hxx_writeb(usb_mode, MB86HXX_USB0_MODE);
	} else if((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base1 ){
    	if ( pDmaChannel->transmit ) {
			/* Paced EP Write Channel */
			usb_mode1 &= 0xe7;
			if(desired_mode)
				usb_mode1 |= pDmaChannel->epnum<<3;
    	} else {
			/* Paced EP Read Channel */
			usb_mode1 &= 0x9f;
			if(desired_mode)
				usb_mode1 |= pDmaChannel->epnum<<5;
    	}
		mb86hxx_writeb(usb_mode1, MB86HXX_USB1_MODE);
	}

	DBG(7, "usb_mode %x %x\n", usb_mode, usb_mode1);
#endif
	spin_unlock_irqrestore(&musb->lock, flags);

}


static void dma_callback(FAPI_DMA_RequestT* pDmaRequest)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
    struct musb_dma_channel *pImplChannel =(struct musb_dma_channel*)pDmaRequest->optData;
#else
    struct musb_dma_channel *pImplChannel =(struct musb_dma_channel*)pDmaRequest->finishOptDataPtr;
#endif
	struct musb *musb = pImplChannel->controller->private_data;
    u8  bytes;
    u8  ep    = pImplChannel->epnum;
    u32 addr;
    u32 data;
    u8  ii; 
    u8 *mbase = pImplChannel->controller->base;
    u32 length;
	unsigned long flags = 0;
	u8 ep_index;

	spin_lock_irqsave(&musb->lock, flags);

	ep_index = musb_readb(mbase, MUSB_INDEX);
	musb_ep_select(mbase, ep);

	pImplChannel->channel.private_data = pImplChannel;
    /* Count transfer length */
    if ( pDmaRequest->result == 0 )
        pImplChannel->channel.actual_len += pDmaRequest->channelConfig.chLength;
     
    /* Copy remaining bytes */
    bytes = pImplChannel->len & DMA_LONG_WORD_MASK;     

    if ( bytes>0 ) {
        addr = pImplChannel->offsetAddress + pImplChannel->channel.actual_len;
        if ( pImplChannel->transmit ) {
            data = MUSB_DMA_READ_ADDR(addr);
            if(bytes <4){
                for( ii=0; ii<bytes; ii++ ) { 
                    musb_writeb(mbase, 0x20+(ep<<2), (data>>(ii<<3))&0xff); 
                }
            }else{
                for( ii=0; ii<4; ii++ ) { 
                    musb_writeb(mbase, 0x20+(ep<<2), (data>>(ii<<3))&0xff); 
                }
                data = MUSB_DMA_READ_ADDR(addr+4);
                for( ii=0; ii<(bytes-4); ii++ ) { 
                    musb_writeb(mbase, 0x20+(ep<<2), (data>>(ii<<3))&0xff); 
                }
           }
        } else {
            data = 0;
            if(bytes <4){
                for( ii=0; ii<bytes; ii++ ) { 
                    data |= musb_readb(mbase, 0x20+(ep<<2))<<(ii<<3);
                }
                MUSB_DMA_WRITE_ADDR(addr,data);
            }else{
                for( ii=0; ii<4; ii++ ) { 
                    data |= musb_readb(mbase, 0x20+(ep<<2))<<(ii<<3);
                }
                MUSB_DMA_WRITE_ADDR(addr,data);
                data = 0;
                for( ii=0; ii<(bytes-4); ii++ ) { 
                    data |= musb_readb(mbase, 0x20+(ep<<2))<<(ii<<3);
                }
                MUSB_DMA_WRITE_ADDR(addr+4,data);
            }
        }
        
        pImplChannel->channel.actual_len += bytes; 
    }
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
		if(mbase == usb_base){
			up(&dma_sem);
		}else if(mbase == usb_base1 ){
			up(&dma_sem1);
		}
#endif
    
    /* Switch to next sub transfer or finish */
    if ( pDmaRequest->result == 0  && 
         pImplChannel->channel.actual_len != pImplChannel->len
    ) {

        /* set length of next (sub)transfer */
		if(pImplChannel->max_packet_sz < (pImplChannel->len - pImplChannel->channel.actual_len))
        	length = pImplChannel->max_packet_sz & ~DMA_LONG_WORD_MASK;
		else
        	length = (pImplChannel->len - pImplChannel->channel.actual_len) & ~DMA_LONG_WORD_MASK;

        pDmaRequest->channelConfig.chLength = length;

        /* set start address of next (sub)transfer */
        if ( pImplChannel->transmit ) {
            pDmaRequest->channelConfig.chRdline_Lines = 
                                        (length-1)/pImplChannel->max_packet_sz+1;
            pDmaRequest->channelConfig.chRdaddr_OffsetAddress = 
                pImplChannel->offsetAddress + pImplChannel->channel.actual_len;
        
        } else {
            pDmaRequest->channelConfig.chWrline_Lines = 
                                        (length-1)/pImplChannel->max_packet_sz+1;
            pDmaRequest->channelConfig.chWraddr_OffsetAddress = 
               pImplChannel->offsetAddress + pImplChannel->channel.actual_len;
        }

        /* send DMA request */
        dma_funcStr.SendRequest( pDmaRequest );
        
    } else {
		/* finished USB-DMA transfer */
		u8 devctl;

		DBG(7, "pDmaRequest %p ep %x transmit %x length %d pImpleChannel %p\n",
		pDmaRequest, ep, pImplChannel->transmit, pImplChannel->len, pImplChannel);

		devctl = musb_readb(mbase, MUSB_DEVCTL);

		pImplChannel->channel.status = MUSB_DMA_STATUS_FREE;

		/* completed */
		if ((devctl & MUSB_DEVCTL_HM)
			&& (pImplChannel->transmit)
			&& ((pImplChannel->channel.desired_mode == 0)
			    || (pImplChannel->channel.actual_len &
			    (pImplChannel->max_packet_sz - 1)))
	    ) {
			u16 txcsr;

			/*
			 * The programming guide says that we
		 	* must clear DMAENAB before DMAMODE.
		 	*/
			musb_ep_select(mbase, ep);
			txcsr = musb_readw(mbase, MUSB_TXCSR);
			txcsr &= ~(MUSB_TXCSR_DMAENAB
					| MUSB_TXCSR_AUTOSET);
			musb_writew(mbase, MUSB_TXCSR, txcsr);
			/* Send out the packet */
			txcsr &= ~MUSB_TXCSR_DMAMODE;
			txcsr |=  MUSB_TXCSR_TXPKTRDY;
			musb_writew(mbase, MUSB_TXCSR, txcsr);
		}
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
		if (!pImplChannel->transmit){
			musb_writew(mbase, MUSB_INTRRXE, musb->epmask & 0xfffe);
		}else{
			u16 int_txe;

			int_txe = musb_readw(mbase, MUSB_INTRTXE);
			musb_writew(mbase, MUSB_INTRTXE, int_txe | (1 << ep));
		}
		if((irq_disabled & 0x0f) && pImplChannel->transmit){
			up(&dma_sem);
		}
#endif /* CONFIG_MACH_FUJITSU_TVSTBSOC */
		/* completed */
		musb_dma_completion(
			pImplChannel->controller->private_data,
			ep,
			pImplChannel->transmit);
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
		if (!pImplChannel->transmit){
			musb_writew(mbase, MUSB_INTRRXE, musb->epmask & 0xfffe);
		}
#endif
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
		if((pImplChannel->transmit == 1) && irq_disabled)
			irq_disabled--;
		else if(!pImplChannel->transmit && irq_disabled)
			irq_disabled -= 0x10;

		if(!irq_disabled ){
			mb86hxx_writeb(0x7e, MB86HXX_USB_INT_MASK);
		}
#endif /* CONFIG_MACH_FUJITSU_TVSTBSOC */
    }
	/* restore index */
	musb_writeb(mbase, MUSB_INDEX, ep_index);

	spin_unlock_irqrestore(&musb->lock, flags);

}

static void dma_request_send(struct musb_dma_channel *pImplChannel, FAPI_DMA_RequestT* pDmaRequest, dma_addr_t dma_addr, u32 length)
{
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC1
	unsigned long mbase = (unsigned long)pImplChannel->controller->base;
#endif
    pImplChannel->offsetAddress     = IO_ADDRESS((uint32_t)dma_addr);

    pDmaRequest->preEnableCallback  = NULL;
    pDmaRequest->postEnableCallback = NULL;
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
    pDmaRequest->postFinishCallback = dma_callback;
    pDmaRequest->optHandle          = NULL;
    pDmaRequest->optData            = pImplChannel;
    pDmaRequest->optFinishCallback  = NULL;
    pDmaRequest->optChangeCallback  = NULL;
    pDmaRequest->channelConfig.chRdaddr_EndianSwap  = 0;
    pDmaRequest->channelConfig.chWraddr_EndianSwap = 0;
#else
    pDmaRequest->finishCallback     = (FAPI_DMA_CallbackT)dma_callback;
    pDmaRequest->postEnableOptDataPtr = pImplChannel;
    pDmaRequest->finishOptDataPtr   = pImplChannel;
    pDmaRequest->changeCallback     = NULL;
    pDmaRequest->channelConfig.chConfig_RdEndianSwap = 0;
    pDmaRequest->channelConfig.chConfig_WrEndianSwap = 0;
#endif
    pDmaRequest->channelConfig.chConfig = 0;
    pDmaRequest->channelConfig.chLladdr = 0;
    
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
    pDmaRequest->preEnableCallback  = dma_prepare;
#else
    pDmaRequest->preEnableCallback  = (FAPI_DMA_CallbackT)dma_prepare;
#endif

    if ( pImplChannel->transmit ) {
        pDmaRequest->channelConfig.chRdaddr_OffsetAddress      = 0;
        pDmaRequest->channelConfig.chRdline_LoopIncrement      = 0;
        pDmaRequest->channelConfig.chRdlpaddr_LoopAddress      = 0;
        pDmaRequest->channelConfig.chRdinc_LineIncrement       = 0;
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        pDmaRequest->channelConfig.chRdaddr_PeripheralAddress  = 0;
        pDmaRequest->channelConfig.chWraddr_PeripheralAddress  = 0x7;	/* USB */
        pDmaRequest->channelConfig.chWrinc_LineLength          = 4;
#else
        pDmaRequest->channelConfig.chConfig_RdPeripheralAddress  = 0;
        if((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base )
            pDmaRequest->channelConfig.chConfig_WrPeripheralAddress  = 0x7;	/* USB */
        else
            pDmaRequest->channelConfig.chConfig_WrPeripheralAddress  = 0xE;	/* USB1 */

        pDmaRequest->channelConfig.chConfig_Read64             = 0;
        pDmaRequest->channelConfig.chWrinc_LineLength          = 8;
        pDmaRequest->channelConfig.chConfig_Write64            = 1;  // 64bit case 
#endif
        pDmaRequest->channelConfig.chWraddr_OffsetAddress      = MUSB_FIFO_ADDR(pImplChannel->epnum);
        pDmaRequest->channelConfig.chWrline_LoopIncrement      = 0;
        pDmaRequest->channelConfig.chWrline_Lines              = 0;
        pDmaRequest->channelConfig.chWrinc_LineIncrement       = 0;
        pDmaRequest->channelConfig.chWrlpaddr_LoopAddress      = MUSB_FIFO_ADDR(pImplChannel->epnum);
        pDmaRequest->channelConfig.chRdinc_LineLength          = pImplChannel->max_packet_sz;
    } else {
#ifdef CONFIG_MACH_FUJITSU_TVSTBSOC
        pDmaRequest->channelConfig.chRdaddr_PeripheralAddress  = 0x7;	/* USB */
        pDmaRequest->channelConfig.chWraddr_PeripheralAddress  = 0;
        pDmaRequest->channelConfig.chRdinc_LineLength          = 4;
#else
        if((mbase & MB86HXX_USB_BASE_MASK) == (unsigned long)usb_base )
            pDmaRequest->channelConfig.chConfig_RdPeripheralAddress  = 0x7;	/* USB */
        else
            pDmaRequest->channelConfig.chConfig_RdPeripheralAddress  = 0xE;	/* USB1 */

        pDmaRequest->channelConfig.chConfig_WrPeripheralAddress  = 0;
        pDmaRequest->channelConfig.chConfig_Write64            = 0;
        pDmaRequest->channelConfig.chRdinc_LineLength          = 8;
        pDmaRequest->channelConfig.chConfig_Read64             = 1;  //64bit case
#endif
        pDmaRequest->channelConfig.chRdaddr_OffsetAddress      = MUSB_FIFO_ADDR(pImplChannel->epnum);
        pDmaRequest->channelConfig.chRdline_LoopIncrement      = 0;
        pDmaRequest->channelConfig.chRdline_Lines              = 0;
        pDmaRequest->channelConfig.chRdinc_LineIncrement       = 0;
        pDmaRequest->channelConfig.chRdlpaddr_LoopAddress      = MUSB_FIFO_ADDR(pImplChannel->epnum);
        pDmaRequest->channelConfig.chWraddr_OffsetAddress      = 0;
        pDmaRequest->channelConfig.chWrline_LoopIncrement      = 0;
        pDmaRequest->channelConfig.chWrlpaddr_LoopAddress      = 0;
        pDmaRequest->channelConfig.chWrinc_LineIncrement       = 0;
        pDmaRequest->channelConfig.chWrinc_LineLength          = pImplChannel->max_packet_sz;
    }

    /* set length of next transfer */
    pDmaRequest->channelConfig.chLength = length & ~DMA_LONG_WORD_MASK;

    /* set start address of next (sub)transfer */
    if ( pImplChannel->transmit ) {
        pDmaRequest->channelConfig.chRdline_Lines = 
                                        (length-1)/pImplChannel->max_packet_sz+1;
        pDmaRequest->channelConfig.chRdaddr_OffsetAddress = 
               pImplChannel->offsetAddress + pImplChannel->channel.actual_len;
    }else {
        pDmaRequest->channelConfig.chWrline_Lines = 
                                        (length-1)/pImplChannel->max_packet_sz+1;
        pDmaRequest->channelConfig.chWraddr_OffsetAddress = 
               pImplChannel->offsetAddress + pImplChannel->channel.actual_len;
    }
    /* send DMA request */
    dma_funcStr.SendRequest( pDmaRequest );
}

#endif /* CONFIG_ARCH_MB86HXX */

